// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.components;

import org.apache.tapestry.BaseComponent;

import java.util.Date;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.security.NoSuchAlgorithmException;


import cexer.*;
import cexer.util.FingerprintHelper;
import cexer.util.Logging;

public abstract class Applet extends BaseComponent {

    public abstract Course getCourse();

    public abstract Round getRound();

    public abstract Exercise getExercise();

    public abstract Student getStudent();

    public abstract String getUserid();

    public abstract String getLang();
    
    public abstract StudentSession getStudentSession();

    public String getExerciseNumber() {
    	int course = getCourseId();
        int round = getRound().getRoundNumber();
        int exercise = getExercise().getNumber();
        return getCourseId() + "." + round + "." + exercise;
    }

    /**
     * Gets the additional applet parameters for the current course (or for the
     * dummy course if the user is testing the exercise from the monitor interface),
     * round and exercise.
     * @return
     */
    public List getAdditionalAppletParameters() {
        Hashtable parameters = new Hashtable();

        // course parameters
        Iterator iter = getCourse().getAdditionalAppletParameters().iterator();
        if(getStudentSession().getUsingMonitorUI()) {
            DataStorage ds = DataStorage.getInstance();
            Course tempCourse = ds.getCourse(ds.DUMMY_COURSE);
            iter = tempCourse.getAdditionalAppletParameters().iterator();
        }

        while(iter.hasNext()) {
            AppletParameter param = (AppletParameter)iter.next();
            parameters.put(param.getName(), param);
        }

        //round parameters
        iter = getRound().getAdditionalAppletParameters().iterator();
        while(iter.hasNext()) {
            AppletParameter param = (AppletParameter)iter.next();
            parameters.put(param.getName(), param);
        }

        // exercise specific parameters
        iter = getExercise().getAppletParameters().iterator();
        while(iter.hasNext()) {
            AppletParameter param = (AppletParameter)iter.next();
            parameters.put(param.getName(), param);
        }

        return new LinkedList(parameters.values());
    }
    
    public List getLocalizedStrings() {
        String lang = getLang();
        if (lang == null) {
            lang = getPage().getLocale().getLanguage();
        }
        
        return getExercise().getLocalizedStrings(lang, getCourse());
    }

    public String getServerUrl() {
    	return getPage().getRequestCycle().getRequestContext().getRequest().getRequestURL().toString();
    }
    
    /**
     * Gets the course id.
     * @return
     */
    public int getCourseId() {
        if(getStudentSession().getUsingMonitorUI()) {
            return DataStorage.DUMMY_COURSE;
        } else {
            return getCourse().getId();
        }
    }

    public long getRandomSeed() {
    	return new Date().getTime();
    }

    public String getFingerprint() {
        Exercise e = getExercise();
        ExerciseTemplate et = e.getExerciseTemplate();
        List params = e.getAppletParameters();
        String exClass = null;

        for (int i=0; i<params.size(); i++) {
            AppletParameter param = (AppletParameter)params.get(i);
            if (param.getName().equals("ex")) {
                exClass = param.getValue();
                break;
            }
        }

        if (exClass == null) {
            params = getRound().getAdditionalAppletParameters();
            for (int i=0; i<params.size(); i++) {
                AppletParameter param = (AppletParameter)params.get(i);
                if (param.getName().equals("ex")) {
                    exClass = param.getValue();
                    break;
                }
            }
        }

        if (exClass == null) {
            params = getCourse().getAdditionalAppletParameters();
            for (int i=0; i<params.size(); i++) {
                AppletParameter param = (AppletParameter)params.get(i);
                if (param.getName().equals("ex")) {
                    exClass = param.getValue();
                    break;
                }
            }
        }

        String fingerprint = FingerprintHelper.getFingerprint("" + getCourse().getId(),
                            getRound().getRoundNumber(),
                            e.getNumber(), getStudent().getId(),
                            e.getMax(), exClass);
        
        try {
            return FingerprintHelper.calculateMD5(fingerprint);
        } catch (NoSuchAlgorithmException exp) {
            exp.printStackTrace();
            Logging.error(this, "Exception calculating checksum", exp);
            return fingerprint;
        }
    }

}
