// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.components;


import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;

import org.apache.tapestry.BaseComponent;
//v4 import org.apache.tapestry.IBinding;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.RedirectException;
import org.apache.tapestry.util.StringSplitter;
import org.apache.tapestry.engine.IEngineService;
import org.apache.tapestry.engine.ILink;
import org.apache.hivemind.Messages;

import cexer.Student;
import cexer.Course;
import cexer.StudentSession;
import cexer.DataStorage;
import cexer.util.LocaleSelectionModel;
import cexer.util.Trakla2SessionListener;



public abstract class Border extends BaseComponent {

    /*
     * Tapesty 4 application state object. Fetches the injected student-session
     * object (see Login.page)
     */
    public abstract StudentSession getStudentSession();

    
    public String getRefresh() {
        // Make sure the properties are loaded
        DataStorage.getInstance();
        return "<meta HTTP-EQUIV=\"refresh\" CONTENT=\"0;URL=http://" + System.getProperty("http.server") + "/app\"/>";
    }

    public void externalLink(IRequestCycle cycle) {
        // Make sure the properties are loaded
        DataStorage.getInstance();
        Object[] parameters = cycle.getListenerParameters();
        String page = ((String) parameters[0]).trim();
        StringSplitter split = new StringSplitter(',');
        String[] pageInfo = split.splitToArray(getMessage(page));
        String message = pageInfo[0];
        if (!message.startsWith("http://")) {
            message = "http://" + System.getProperty("http.server") + "/"+ message;
        }
        throw new RedirectException(message);
    }
    
    public void loginPage(IRequestCycle cycle) {
        String nextPage = "Login";
        if (cycle.getInfrastructure().getRequest().getActivationPath().equals("/monitor")) {
            nextPage = "MonitorLogin";
        }
        cycle.forgetPage(cycle.getPage().getPageName());
        cycle.activate(nextPage);
        return;
    }

    /**
     * Temporary logout-hack.
     * Calls restartService to destroy the session.
     */
    public ILink logout(IRequestCycle cycle) {
        StudentSession session = getStudentSession(); 
        String logoutUrl = session.getLogoutUrl();
        Student student = session.getStudent();
        
        // Get student ID for logging
        String studentId = null;
        if (student != null) {
            studentId = session.getStudent().getId();
        }
        
        // Kill session
        session.setIsLoggedIn(false);
        session.closeSession();
        cycle.forgetPage(cycle.getPage().getPageName());        

        // Redirect to an appropriate page
        if (logoutUrl != null && logoutUrl.length() > 0) {
            // Shibboleth
            // cexer.util.Logging.info(this, "Redirecting to logout url " + logoutUrl);
            cexer.util.Logging.info(this, "Logged out " + studentId + " (shibboleth)");
            throw new RedirectException(logoutUrl);
        } else {
            // Traditional
            //cexer.util.Logging.info(this, "Using traditional logout");
            cexer.util.Logging.info(this, "Logged out " + studentId + " (traditional)");
            return getRestartService().getLink(false, null);
        }
    }
    
    /**
     * Get a reference to the restart-service. Used by the logout-link.
     * @return reference to the restart-service
     */
    public abstract IEngineService getRestartService();
  

    
    public void localization(IRequestCycle cycle) {
        if (cycle.getInfrastructure().getRequest().getActivationPath().equals("/monitor")) {
            cycle.setListenerParameters(new String[] { getPage().getPageName(),"monitor" });
        } else {
            cycle.setListenerParameters(new String[] { getPage().getPageName(),"app" });
        }

        cycle.activate("LanguageChange");
    }

    public LocaleSelectionModel getLocaleModel() {
        return DataStorage.getInstance().getLocaleSelectionModel();
    }

    
    /**
     * Asks the framework if the Visit-object exists, e.g. if the HttpSession has been created
     * or if the current session is stateless.
     * @return TRUE if the StudentSession-object is created, otherwise FALSE
     */
    public abstract boolean getSessionState();
    
    public boolean isLogonTrue() {
        if(!getSessionState()) {
            //System.out.println("isLogonTrue(): getSessionState() false");
            return false;
        }
        StudentSession session = getStudentSession();
        if (session == null) {
            //System.out.println("isLogonTrue(): session == null");
            return false;
        }
        if (!session.getIsLoggedIn()) {
            //System.out.println("isLogonTrue(): session.getIsLoggedIn() == false");
            return false;
        }
        if (session.getUserId() == null) {
            //System.out.println("isLogonTrue(): session.getUserId() == null");
            return false;
        }
        if (session.getStudent() == null) {
            //System.out.println("isLogonTrue(): session.getStudent() == null");
            return false;
        }
        //System.out.println("isLogonTrue(): OK! session.getUserId() = " + session.getUserId());
        return true;
    }
    
    /**
     * Returns true if login is not needed
     */
    public boolean isLogonOk() {
        String pageName = getPage().getPageName();

        boolean answer = isLogonTrue()
            || pageName.equals("Login")
            || pageName.equals("LoginShib")
            || pageName.equals("Register")
            || pageName.equals("Home")
            || pageName.equals("MonitorLogin")
            || pageName.equals("ForgotPassword") 
            || pageName.equals("TestRegister")
            || pageName.equals("Feedback")
            || pageName.equals("Material")
            || pageName.equals("RecoverPassword")
            || pageName.equals("Help")
            || pageName.equals("Error");

        //System.out.println("isLogonOk(): page: " + getPage().getPageName() + " status: " + answer);        
        return (answer);
    }

    public void languageLink() {
        IRequestCycle cycle = this.getPage().getEngine().getInfrastructure().getRequestCycle();
        
        cycle.getEngine().setLocale(new Locale((String) cycle.getListenerParameters()[0], ""));
        cycle.cleanup();
        cycle.activate(getPage().getPageName());
        
        // Jump to another page to make the language change take effect.
        // This is now unnecessary due to the cleanup() and activate() above,
        // and can be removed after testing.
        //localization(cycle);
    }
    
    public boolean isHasEbook() {
        String url = getEbookUrl();
        return url != null && url.length() > 0;
    }
    
    public String getEbookUrl() {
        StudentSession session = getStudentSession();
        Course course = session.getCourse();
        
        if (course != null) {
            return course.getEbookUrl(getPage().getLocale().getLanguage());
        } else {
            return "";
        }
    }
    
    /**
     * Get the number of active sessions on the server. Both monitor- and student-sessions are included in this count.
     * @return number of sessions
     */
    public int getNumberOfUsersOnline() {
        return Trakla2SessionListener.getNumberOfSessions();
    }
    
    public String getServerName() {
        DataStorage.getInstance();
        return System.getProperty("http.server");
    }
    
    public String getServerTime() {
        DateFormat df = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
        return df.format(new Date());
    }


    public String getGoogleScript() {
        // Make sure the properties are loaded
        DataStorage.getInstance();
        
        String userId = System.getProperty("googleId");
        
        if (userId == null || userId.length() <= 0) {
            return "";
        }
        
        String script = "<script src='https://ssl.google-analytics.com/urchin.js' type='text/javascript'></script>\n";
        script += "<script type='text/javascript'>\n";
        script += "_uacct = '" + userId + "';\n";
        script += "urchinTracker();\n";
        script += "</script>\n";

        return script;
    }
    
    
    /**
     * Returns a list of available languages (two-letter codes) 
     */
    public String[] getLanguages() {
        String rawList = getMessages().getMessage("languages");
        
        if (rawList != null) {
            return rawList.split(",");
        } else {
            cexer.util.Logging.error(this, "Available languages are not specified");
            return new String[]{"en"};
        }
    }

    public String getLanguageName() {
        return getMessages().getMessage("languages." + getLang());
    }
    
    public abstract void setLang(String lang);
    public abstract String getLang();
}
