package cexer.components;

import cexer.*;
import cexer.pages.*;
import cexer.util.LocaleSelectionModel;
import cexer.util.Lock;
import cexer.util.Logging;
import cexer.util.chart.ChartProvider;
import cexer.util.chart.ChartAsset;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IPage;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.IAsset;


import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Iterator;
//import java.util.ListIterator;
import java.util.Locale;

import java.awt.Paint;

import org.jCharts.Chart;
import org.jCharts.chartData.DataSeries;
import org.jCharts.chartData.AxisChartDataSet;
import org.jCharts.properties.AreaChartProperties;
import org.jCharts.properties.ChartProperties;
import org.jCharts.properties.AxisProperties;
import org.jCharts.properties.LegendProperties;
import org.jCharts.types.ChartType;
import org.jCharts.axisChart.AxisChart;
import org.jCharts.chartData.ChartDataException;
import org.jCharts.properties.StackedBarChartProperties;



public abstract class CourseDetails extends BaseComponent implements ChartProvider {

    private IAsset pointDistributionImageAsset; // persistent? 
    private IAsset gradeDistributionImageAsset; // persistent?

    public static String POINT_DISTRIBUTION = "points";
    public static String GRADE_DISTRIBUTION = "grades";

    /**
     * Get a list of supported locales on the currently selected course
     * @return
     */
    public Iterator getLocales() {
        return getMonitorSession().getCourse().getSupportedLocaleIterator();
    }

    public abstract String getDescriptionLocale();

    public abstract void setDescriptionLocale(String descriptionLocale);

    /**
     * Gets the current course description for the editing textarea.
     * @return description
     */
    public String getDescription() {
        return getCourse().getLocalizedDescription(getDescriptionLocale());
    }

    /**
     * Saves the changed description to the session, for persistence over form rewind. The description is saved 
     * to the database by <b>modifyDescription</b>
     * @param description
     */
    public void setDescription(String description) {
        setModifiedDescription(description);
    }

    public String getName() {
        return getCourse().getLocalizedName(getDescriptionLocale());
    }
   
    public void setName(String name) {
        setModifiedName(name);
    }

    public String getHomepage() {
        return getCourse().getLocalizedLink(getDescriptionLocale());
    }
   
    public void setHomepage(String url) {
        setModifiedHomepage(url);
    }

    public String getEbookUrl() {
        return getCourse().getEbookUrl(getDescriptionLocale());
    }
   
    public void setEbookUrl(String url) {
        setModifiedEbookUrl(url);
    }

    public String getDateFormat() {
        return getCourse().getDateformat(getDescriptionLocale());
    }
   
    public void setDateFormat(String formatString) {
        setModifiedDateFormat(formatString);
    }

    
    public boolean isEditingDescription() {
        if(getIndex() == getEditingIndex()) {
            return true;
        } else {
            return false;
        }
    }


    /**
     * Activates the coursedescription editing mode
     * @param cycle
     */
    public void editDescription(IRequestCycle cycle) {
        Object[] params = cycle.getListenerParameters();
        DataStorage ds = DataStorage.getInstance();
        
        // Set warning if lock is active
        Lock l = ds.addLock(new Integer(getCourse().getId()), getMonitorSession());
        if (l == null) {
            setLockWarning(true);
        }
        
        setEditingIndex(((Integer)params[0]).intValue());
        setEditingLocale((String)params[1]);
        //cycle.activate(this);
    }
    
    /**
     * Activates the Help editing page
     * @param cycle
     */
    public IPage editHelp(IRequestCycle cycle, int index, String descriptionLocale) {
        MonitorEditHelpPage page = (MonitorEditHelpPage)cycle.getPage("MonitorEditHelp");
        
        page.setTheLocale(descriptionLocale);
        page.setCourseId(getCourse().getId());
        //setEditingIndex(((Integer)params[0]).intValue());
        //setEditingLocale((String)params[1]);
        
        return page;
        
    }

    /**
     * Ends the coursedescription editing mode without saving changes
     * @param cycle
     */
    public void endEditDescription(IRequestCycle cycle) {
        releaseLock();
        setEditingIndex(-1);
        setEditingLocale(null);
        //cycle.activate(getPage());
    }


    /**
     * Saves a changed course description
     * @param cycle
     */
    public void modifyDescription(IRequestCycle cycle) {
        if (!hasLock()) {
            setLockWarning(true);
        }
        
        DataStorage ds = DataStorage.getInstance();
        
        String name = getModifiedName();
        if (name == null) {
            name = "";
        }
        
        String homepage = getModifiedHomepage();
        if (homepage == null) {
            homepage = "";
        }
        
        String ebookUrl = getModifiedEbookUrl();
        if (ebookUrl == null) {
            ebookUrl = "";
        }
            
        String dateFormat = getModifiedDateFormat();
        if (dateFormat == null) {
            dateFormat = "";
        }

        String description = getModifiedDescription();
        if (description == null) {
            description = "";
        }
        
        
        boolean success = ds.updateCourseDescription(getCourse().getId(), getEditingLocale(), name, homepage, ebookUrl, dateFormat, description);
        if (success) {
            getCourse().addLocalizedData(getEditingLocale(), name, description, homepage, dateFormat, ebookUrl);
        }

        releaseLock();
        setEditingIndex(-1);
    }

    public Object[] createParameterArray(int parameter1, Object parameter2) {
        Object[] tempArray = new Object[2];
        tempArray[0] = new Integer(parameter1);
        tempArray[1] = parameter2;
        return tempArray;
    }

    /**
     * @return true if the course has a specified organizer
     */
    public boolean getHasOrganizer() {
        return getCourse().getOrganizer() != null;
    }

    /**
     * @return true if the course has a specified organization
     */
    public boolean getHasOrganization() {
        return getCourse().getOrganization() != null;
    }

    /**
     * Releases a locked exercise description.
     * @return
     */
    private boolean releaseLock() {
        return checkLock(true);
    }

    private boolean checkLock(boolean b) {
        DataStorage ds = DataStorage.getInstance();
        MonitorSession session = getMonitorSession();
        Integer cid = new Integer(session.getCourse().getId());
        synchronized(this) {
            Lock l = ds.renewLock(cid, getMonitorSession());
            if (l == null) {
                return false;
            }
            
            if (l.getId() == session) {
                if (b) {
                    ds.removeLock(cid);
                }
                return true;
            }
        }
        return false;
    }

    private boolean hasLock() {
        return checkLock(false);
    }

    public String getLocalizedCourseName() {
        return getCourse().getLocalizedName(getPage().getLocale().getLanguage());
    }

    /**
     * Adds a empty description in the selected locale
     * @param cycle
     */
    public void addLocale(IRequestCycle cycle) {
        if(getLocaleToAdd() == null || getLocalizedCourseNameToAdd() == null) {
            return;
        }
        
        DataStorage ds = DataStorage.getInstance();
        boolean success = ds.addCourseData(getLocalizedCourseNameToAdd(), getLocaleToAdd().getLanguage(), getMonitorSession().getCourse().getId());

        if(success) {
            getMonitorSession().getCourse().addLocalizedData(getLocaleToAdd().getLanguage(), getLocalizedCourseNameToAdd(), "", "", "dd MMM. yyyy HH:mm:ss", "");
        }
    }

    /**
     * Tells if the course is missing a description in one (or more) of the locales specified in
     * DataStorage by getLocaleSelectionModel
     * @return true if course doesn't have descriptions in all available locales
     */
    public boolean isMissingLocales() {
        Iterator i = getLocales();
        DataStorage ds = DataStorage.getInstance();
        int size = 0;
        while(i.hasNext()) {
            size++;
            i.next();
        }
        if(size < ds.getLocaleSelectionModel().getOptionCount()) {
            return true;
        }
        return false;
    }
    
    
    /**
     * Returns a LocaleSelectionModel containing the locales in which the course doesn't have a description.
     * @return LocaleSelectionModel with the missing locales
     */
    public LocaleSelectionModel getMissingLocaleModel() {
        Iterator i = getLocales();
        DataStorage ds = DataStorage.getInstance();
        List missingLocales = ds.getLocales();
        //System.out.println("ds.getLocales().size(): " + missingLocales.size());
        while(i.hasNext()) {
            missingLocales.remove(new Locale((String)i.next()));
        }
        return new LocaleSelectionModel(missingLocales);
    }


    public String getPointsForGrade() {
        int[] gradeMinPoints = getCourse().getGradeMinPoints();
        int index = getIndex();
        if (index < gradeMinPoints.length - 1) {
            return gradeMinPoints[index] + " - " + gradeMinPoints[index+1];
        } else {
            return gradeMinPoints[index] + " - " + getMonitorSession().getCourse().getMax();
        }
    }

    /**
     * @return true if the course has grading limits.
     */
    public boolean isGradeLimits() {
        if(getCourse().getGrades().length == 0 || getCourse().getGradeMinPoints().length == 0) {
            return false;
        } else {
            return true;
        }
    }

    public String getGrade() {
        return getCourse().getGrade(getMinPoints());
    }

    /**
     * @return Point distribution image asset
     */
    public IAsset getPointDistributionImage() {
        if (pointDistributionImageAsset == null) {
            pointDistributionImageAsset = new ChartAsset(getPage().getRequestCycle(), this, POINT_DISTRIBUTION);
        }
        return pointDistributionImageAsset;
    }

    /**
     * @return Grade distribution image asset
     */
    public IAsset getGradeDistributionImage() {
        if (gradeDistributionImageAsset == null) {
            gradeDistributionImageAsset = new ChartAsset(getPage().getRequestCycle(), this, GRADE_DISTRIBUTION);
        }
        return gradeDistributionImageAsset;
    }

    public Chart getChart(String chartName) throws ChartDataException {
        if (GRADE_DISTRIBUTION.equals(chartName)) {
            return getGradeDistributionChart();
        } else { // the default functionality, includes GRADE_DISTRIBUTION
            return getPointDistributionChart();
        }
    }

    private Chart getGradeDistributionChart() throws ChartDataException {
        // data for grade distribution.
        Course c = getCourse();
        int[] minPoints = c.getGradeMinPoints();
        // we add grade distribution only if the data is available
        if(minPoints != null && minPoints.length > 0) {
        // info for dataseries
            String xtitle = "grades";
            String ytitle = "Number of Students";
            String title = "Distribution of grades";
            String[] grades = c.getGrades();
            String[] labels = new String[grades.length+1];
            labels[0] = "0";

            for(int j = 1; j < labels.length; j++) {
                labels[j] = grades[j-1];
            }

            DataSeries series = new DataSeries(labels,xtitle,ytitle,title);
            List students = getStudentResults();
            int[] gradeDistribution = new int[grades.length+1];
            Iterator i = students.iterator();

            // get the grades for all students on the course and 
            // group the same grades together
            while(i.hasNext()) {
                Student s = (Student)i.next();
                String grade = s.getGrade(c);
                if(grade.equals("0")) {
                    gradeDistribution[0]++;
                    continue;
                }

                for(int j = 0; j < grades.length; j++) {
                    if(grade.equals(grades[j])) {
                        gradeDistribution[j+1]++;
                        break;
                    }
                }
            }

            // create data for grade distribution and put the
            // grade bars to the chart. Each grade bar is in the
            // middle of the points that need to be gained for that
            // grade.
            double[][] data = new double[1][gradeDistribution.length];
            for(int j = 0; j <= grades.length; j++) {
                data[0][j] = gradeDistribution[j];
            }

            // create properties, etc.
            String[] labelTitles = new String[1];
            labelTitles[0] = "Grade";
            Paint[] paints = new Paint[] { java.awt.Color.red };
            StackedBarChartProperties barChartProperties = new StackedBarChartProperties();
            AxisChartDataSet axisData = new AxisChartDataSet(data, labelTitles,paints,ChartType.BAR,barChartProperties);
            series.addIAxisPlotDataSet(axisData);
            return createActualChart(series, 400, 300, false);
        }
        return null;
    }

    private Chart getPointDistributionChart() throws ChartDataException {
        Integer[] pointAmounts = getStatistics().getPointAmounts();

        // establish the length of the Xaxis.
        int length = pointAmounts.length;
        if(getCourse().getMax() > pointAmounts.length) {
            length = getCourse().getMax();
        }
        String[] labels = new String[length];

        for(int i = 0; i < labels.length;i++) {
            labels[i] = "" + i;
        }

        // info for dataseries
        String xtitle = "Points";
        String ytitle = "Number of Students";
        String title = "Distribution of Results";
        DataSeries series = new DataSeries(labels,xtitle,ytitle,title);

        // data for point distribution
        double[][] data = new double[1][length];
        for(int i = 0; i < pointAmounts.length;i++) {
            data[0][i] = pointAmounts[i].doubleValue();
        }

        String [] labelTitles = { "Points" };
        Paint[] paints = { java.awt.Color.blue };
        AreaChartProperties aproperties = new AreaChartProperties();
        AxisChartDataSet axisData = new AxisChartDataSet(data,labelTitles,paints,ChartType.AREA,aproperties);
        series.addIAxisPlotDataSet(axisData);

        // chart creation itself
        return createActualChart(series);
    }


    private Chart createActualChart(DataSeries series) {
        return createActualChart(series, 800, 600, false);
    }

    private Chart createActualChart(DataSeries series, int width, int height, boolean horizontal) {
        ChartProperties chartProperties= new ChartProperties();
        chartProperties.setEdgePadding(20f);

        // FIXME: this does not compile -TA
        //AxisProperties axisProperties= new AxisProperties(horizontal);
        //axisProperties.getYAxisProperties().setShowGridLines(AxisTypeProperties.GRID_LINES_ALL);
        //axisProperties.getYAxisProperties().setShowTicks(AxisTypeProperties.TICKS_ALL);
        AxisProperties axisProperties = new AxisProperties();

        LegendProperties legendProperties= new LegendProperties();
        legendProperties.setNumColumns(2);

        Chart result = new AxisChart( series, chartProperties, axisProperties, legendProperties, width, height );
        return result;
    }

    public Chart getChart() throws ChartDataException {
        return getChart(null);
    }

    public List getStudentResults() {
        return getMonitorSession().getStudentResults();
    }
    
    public Statistics getStatistics() {
        return  getMonitorSession().getStatistics();
    }

    public abstract String getModifiedName();
    public abstract void setModifiedName(String name);
    
    public abstract String getModifiedHomepage();
    public abstract void setModifiedHomepage(String url);
    
    public abstract String getModifiedEbookUrl();
    public abstract void setModifiedEbookUrl(String url);
    
    public abstract String getModifiedDateFormat();
    public abstract void setModifiedDateFormat(String formatString);
    
    public abstract String getModifiedDescription();
    public abstract void setModifiedDescription(String description);
    
    public abstract void setIndex(int i);

    public abstract int getIndex();
    
    public abstract int getEditingIndex();
    
    public abstract void setEditingIndex(int index);

    public abstract String getEditingLocale();
    
    public abstract void setEditingLocale(String locale);  

    public abstract MonitorSession getMonitorSession();

    public abstract Course getCourse();

    public abstract void setCourse(Course c);
    
    public abstract Locale getLocaleToAdd();
    
    public abstract boolean isLockWarning();
    public abstract void setLockWarning(boolean value);
    

    /**
     * Get the content of the course name textbox when adding a course
     * description in a new locale
     * @return
     */
    public abstract String getLocalizedCourseNameToAdd();
    

    public int[] getGradeMinPoints() {
        return getCourse().getGradeMinPoints();
    }

    public abstract int getMinPoints();

    public abstract void setMinPoints(int points);

    public MonitorChartPopupPage chartPopup(IRequestCycle cycle, String chartName) {
        IAsset image = null;
        if (chartName.equals("grades")) {
            image = getGradeDistributionImage();
        }

        if (chartName.equals("points")) {
            image = getPointDistributionImage();
        }
        MonitorChartPopupPage nextPage = (MonitorChartPopupPage)cycle.getPage("MonitorChartPopup");
        nextPage.setImage(image);
        nextPage.setName(chartName);
        return nextPage;
    }
    
    public String getFormattedOpen() {
        DateFormat df = new SimpleDateFormat(getCourse().getDateformat(getPage().getLocale().getLanguage()));
        DateFormat df2 = new SimpleDateFormat("Z");
        Calendar cal = new GregorianCalendar();
        cal.setTime(getCourse().getOpen());
        return df.format(cal.getTime()) + " GMT"+ df2.format(cal.getTime());
    }
    
    public String getFormattedClose() {
        DateFormat df = new SimpleDateFormat(getCourse().getDateformat(getPage().getLocale().getLanguage()));
        DateFormat df2 = new SimpleDateFormat("Z");
        Calendar cal = new GregorianCalendar();
        cal.setTime(getCourse().getClose());
        return df.format(cal.getTime()) + " GMT"+ df2.format(cal.getTime());
    }

    /**
     * @return string containing the localized organization name and acronym
     */
    public String getOrganizationName() {
        return getCourse().getOrganization().getLocalizedName(getPage().getLocale().getLanguage()) + " (" + getCourse().getOrganization().getLocalizedAcronym(getPage().getLocale().getLanguage()) + ")";
    }
}
