// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.components;

import cexer.*;
import cexer.util.LocaleSelectionModel;
import cexer.util.Lock;

import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.BaseComponent;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;

public abstract class CourseStatistics extends BaseComponent {

    private Course course;
    private Statistics statistics;
    private Round round;
    private Exercise exercise;
    //private RoundStatistics rs;
    //private ExerciseStatistics es;
    //private boolean[] maximized;
    //private int index;
    private Integer amount;
    private int accumul;
    private NumberFormat nf;
    
    public abstract MonitorSession getMonitorSession();
    
    public abstract int getExerciseIndex();
            
    public abstract String getRowStyle1();
    public abstract String getRowStyle2();
    
    public abstract int getDescriptionIndex();
    
    public abstract void setEditingIndex(int index);
    
    public abstract int getEditingIndex();
    
    public abstract String getEditingLocale();

    public abstract void setEditingLocale(String locale);
    
    public abstract String getDescriptionLocale();    
    
    public abstract String getChangedDescription();
    
    public abstract void setChangedDescription(String description);
    
    public abstract void setEditingRoundNumber(int roundNumber);
    
    public abstract int getEditingRoundNumber();
    
    public abstract Locale getLocaleToAdd();
    
    public abstract String getLocalizedRoundNameToAdd();
    
    public abstract void setLocalizedRoundNameToAdd(String name);
    
    public abstract int getAddRoundDescriptionIndex();
    
    public abstract void setAddRoundDescriptionIndex(int index);
    
    private NumberFormat getNumberFormat() {
	if(nf == null) {
	    nf = NumberFormat.getInstance(getPage().getLocale());
	    nf.setMinimumFractionDigits(0);
	    nf.setMaximumFractionDigits(3);
	}
	return nf;
    }

    public String getMakeupFactor() {
	return getNumberFormat().format(getRound().getMakeupFactor());
    }

    public String getExerciseAverage() {
	return getNumberFormat().format(getExerciseStatistics().getExerciseAverage());
    }

    public String getAverageOfMaximums() {
	return getNumberFormat().format(getExerciseStatistics().getAverageOfMaximums());
    }

    public String getRoundAverage() {
	return getNumberFormat().format(getRoundStatistics().getRoundAverage());
    }

    public Integer getPointAmount() {
	if(getRoundIndex() == 0) {
	    int nr = getAmountOfStudents() - getRoundStatistics().getAmountOfStudents();
	    amount = new Integer(nr);
	    accumul += nr;
	}
	return amount;
    }

    public void setPointAmount(Integer i) {
	amount = i;
	accumul += i.intValue();
    }

    public Exercise getExercise() {
	return exercise;
    }

    public void setExercise(Exercise e) {
	exercise = e;
    }

    public Round getRound() {
	return round;
    }

    public void setRound(Round r) {
	round = r;
	accumul = 0;
    }

    public Course getCourse() {
	return course;
    }

    public void setCourse(Course c) {
	course = c;
    }

    public Statistics getStatistics() {
	return statistics;
    }

    public void setStatistics(Statistics s) {
	statistics = s;
    }

    public RoundStatistics getRoundStatistics() {
        /*RoundStatistics rs = getStatistics().getRoundStatistics(getRound().getRoundNumber()-1);
        if(getRound().getExerciseList().size() != rs.getExercises().size()) {
            DataStorage ds = DataStorage.getInstance();
            setStatistics(ds.getStatisticsForCourse(getCourse().getId()));
            rs = getStatistics().getRoundStatistics(getRound().getRoundNumber()-1);
        }*/
	return getStatistics().getRoundStatistics(getRound().getRoundNumber()-1);
    }

    public void setRoundStatistics(RoundStatistics r) {
	accumul = 0;
	//rs = r;
    }

    public ExerciseStatistics getExerciseStatistics() {
	return getRoundStatistics().getExercise(getExercise().getNumber()-1);
    }

    public void setExerciseStatistics(ExerciseStatistics e) {
    }

    public abstract void setMaximizedRounds(Object o);
    public abstract Object getMaximizedRounds();

    public abstract int getRoundIndex();

    public boolean[] getMaximized() {
	Object o = getMaximizedRounds();
	if(o instanceof boolean[]) {
	    return (boolean[])o;
	}
	return null;
    }

    public boolean isMaximized() {
	boolean[] maximized = getMaximized();
	if(maximized == null)
	    return true;
	if(getRoundIndex() < maximized.length) {
	    return maximized[getRoundIndex()];
	}
	return true;
    }

    public boolean isShowPoints() {
	boolean[] maximized = getMaximized();
	if(maximized == null|| maximized.length == 0)
	    return true;
	return maximized[maximized.length-1];
    }

    public void pointDistribution(IRequestCycle cycle) {
	boolean [] maximized = getMaximized();
	if(maximized != null) {
	    maximized[maximized.length-1] = !maximized[maximized.length-1];
	}
	cycle.activate(getPage());
    }

    public String getExerciseName() {
	return getExercise().getLocalizedName(getPage().getLocale().getLanguage());
    }

    public void maximizeRound(IRequestCycle cycle) {

        Object[] params = cycle.getListenerParameters();
        int parameterIndex = ((Integer)params[0]).intValue();
           
	boolean [] maximized = getMaximized();
	if(maximized != null && parameterIndex < maximized.length) {
	    maximized[parameterIndex] = !maximized[parameterIndex];
	}
	cycle.activate(getPage());
    }

    public int getPercentageOfStudents() {
	float a = (float)getExerciseStatistics().getAmountOfStudents();
	float b = getAmountOfStudents();
	return (int) (100*(a/b));
    }

    public int getAverageFromMaximum() {
	return (int) (100*((float)getExerciseStatistics().getAverageOfMaximums()) / ((float)getExercise().getMax()));
    }

    public int getAverageFromAll() {
	return (int) (100*
		      (((float)getExerciseStatistics().getAverageOfMaximums()) / ((float)getExercise().getMax()))
		       * (((float)getExerciseStatistics().getAmountOfStudents()) / ((float)getAmountOfStudents())));
    }


    public String getLocales() {
	Iterator locales = getExercise().getSupportedLocales().iterator();
  	String ret = "";
  	while(locales.hasNext()) {
  	    ret = ret + locales.next() + " ";
  	}
  	return ret;
    }
    public int getAmountOfStudents() {
	return getStatistics().getNroStudents();
    }

    public Iterator getLocaleIterator() {
        // XXX See addLocale for a remark concerning getRound()
        return getRound().getSupportedLocaleIterator();
    }
    
    
    public void testExercise(IRequestCycle cycle) {
        Object[] params = cycle.getListenerParameters();
	MonitorSession session = getMonitorSession();
    session.setCourse(getCourse());
    session.setRound((Round)session.getCourse().getRoundList().get(((Integer)params[0]).intValue()));
    session.setExercise((Exercise)session.getRound().getExerciseList().get(((Integer)params[1]).intValue()));
	cycle.activate("MonitorTestExercise");
    }

    public void setAccumulatingAmount(int i) {}

    public int getAccumulatingAmount() {
	return accumul;
    }

    public void setAccumulatingPercentage(int i) {}

    public int getAccumulatingPercentage() {
	return (int) (100*((float)accumul / (float)getAmountOfStudents()));	
    }

    public void setPercentage(int i) {}

    public int getPercentage() {
	return (int) (100*(amount.floatValue() / (float)getAmountOfStudents()));
    }

    public String getRowStyle() {
        if((getExerciseIndex() % 2)==1)
            return getRowStyle2();
        else return getRowStyle1();
    }
    
    public Iterator getDescriptionLocales() {
        return getRound().getSupportedLocaleIterator();
    }
    
    public boolean isEditingDescription() {
        if(getDescriptionIndex() == getEditingIndex() && getRoundIndex() == getEditingRoundNumber())
            return true;
        return false;    
    }
    
    public void editDescription(IRequestCycle cycle) {
        Object[] params = cycle.getListenerParameters();
        DataStorage ds = DataStorage.getInstance();
        Round editingRound = (Round)getCourse().getRoundList().get(((Integer)params[0]).intValue());
        Lock l = ds.addLock(new Integer(editingRound.getId()), getMonitorSession());
        if(l == null) {
            cycle.activate(getPage());
            return;
        }
        setEditingRoundNumber(((Integer)params[0]).intValue());        
        setEditingIndex(((Integer)params[1]).intValue());
        setEditingLocale((String)params[2]);
        cycle.activate(getPage());
    }

    public void endEditDescription(IRequestCycle cycle) {
        releaseLock();
        setEditingRoundNumber(-1);        
        setEditingIndex(-1);
        setEditingLocale(null);
        cycle.activate(getPage());
    }
    
    public String getDescription() {
        return getRound().getLocalizedDescription(getDescriptionLocale());
    }
    
    public void setDescription(String description) {
        setChangedDescription(description);
    }
    
    public void modifyDescription(IRequestCycle cycle) {
        DataStorage ds = DataStorage.getInstance();
        Round editingRound = (Round)getCourse().getRoundList().get(getEditingRoundNumber());
        if(getChangedDescription() != null && !getChangedDescription().equals("") && getEditingLocale() != null) {
            if(hasLock()) {
                 /* System.out.println("CourseStatistics: modifyDescription: getChangedDescription: '" 
                        + getChangedDescription() + "' getEditingLocale: '" + getEditingLocale()
                        + "' edtitingRound.getId: '" + editingRound.getId() + "' "); */ 
                boolean success = ds.updateRoundDescription(getChangedDescription(), getEditingLocale(), editingRound.getId());
                if(success) {                   
                    editingRound.setLocalizedDescription(getEditingLocale(), getChangedDescription());
                    //System.out.println("success");
                } else {
                    //setEditError(true);
                    //System.out.println("failure");
                    setEditingIndex(-1);
                    cycle.activate(getPage());
                    return;
                }
                releaseLock();
            }
        }
        setEditingIndex(-1);
        cycle.activate(getPage());        
    }
    
    private boolean releaseLock() {
        return checkLock(true);
        }

        private boolean checkLock(boolean b) {
        DataStorage ds = DataStorage.getInstance();
        MonitorSession session = getMonitorSession();
        Round editingRound = (Round)getCourse().getRoundList().get(getEditingRoundNumber());
        Integer cid = new Integer(editingRound.getId());
        synchronized(this) {
            Lock l = ds.renewLock(cid, getMonitorSession());
            if(l == null) 
            return false;
            if(l.getId() == session) {
            if(b) {
                ds.removeLock(cid);
            }
            return true;
            }
        }
        return false;
        }

        private boolean hasLock() {
        return checkLock(false);
        }
    
    public Object[] createParameterArray(int parameter1, int parameter2, Object parameter3) {
        Object[] tempArray = new Object[3];
        tempArray[0] = new Integer(parameter1);
        tempArray[1] = new Integer(parameter2);
        tempArray[2] = parameter3;
        return tempArray;
    }
    
    public Object[] createParameterArray(int parameter1, int parameter2) {
        Object[] tempArray = new Object[2];
        tempArray[0] = new Integer(parameter1);
        tempArray[1] = new Integer(parameter2);
        return tempArray;
    }
    
        
    public boolean isMissingLocales() {
        Iterator i = getLocaleIterator();
        DataStorage ds = DataStorage.getInstance();
        int size = 0;
        while(i.hasNext()) {
            size++;
            i.next();
        }
        if(size < ds.getLocaleSelectionModel().getOptionCount())
            return true;
        return false;
    }
    
    public LocaleSelectionModel getMissingLocaleModel() {
        Iterator i = getLocaleIterator();
        DataStorage ds = DataStorage.getInstance();
        List missingLocales = ds.getLocales();
        //System.out.println("ds.getLocales().size(): " + missingLocales.size());
        while(i.hasNext()) {
            missingLocales.remove(new Locale((String)i.next()));
        }
        return new LocaleSelectionModel(missingLocales);                                    
    }        
    
    public void refreshAddRoundDescriptionIndex(IRequestCycle cycle, int roundIndex) {
        System.out.println("setAddRoundDescriptionIndex: " + roundIndex);
        setAddRoundDescriptionIndex(roundIndex);
    }
    
    public void addLocale(IRequestCycle cycle) {
        // XXX Adding a localized description to a round requires that the round-attribute isn't null.
        // This is done by saving the round value in a hidden field in the addLocale-form in html.
        // This is not a good solution and can cause some weirdness if it's not taken into account.
        Round round = (Round)getMonitorSession().getCourse().getRoundList().get(getAddRoundDescriptionIndex());
        if(getLocaleToAdd() == null) {
            cycle.activate(getPage());
            return;
        }
        //System.out.println("addLocale: " + getLocaleToAdd() + " roundIndex: " + getAddRoundDescriptionIndex());
        if(getLocalizedRoundNameToAdd() == null)
            setLocalizedRoundNameToAdd("");
        DataStorage ds = DataStorage.getInstance();
        boolean success = ds.addRoundData(getLocalizedRoundNameToAdd(), "", getLocaleToAdd().getLanguage(), round.getId());
        if(success) {
            round.addLocalizedData(getLocaleToAdd().getLanguage(), getLocalizedRoundNameToAdd(), "");
            getRound().addLocalizedData(getLocaleToAdd().getLanguage(), getLocalizedRoundNameToAdd(), "");
        }
        setAddRoundDescriptionIndex(-1);
        cycle.activate(getPage());
    }
    
    public boolean isAddLocale() {
        if(getRoundIndex() == getAddRoundDescriptionIndex())
            return true;
        return false;    
    }
    
    public void setAddLocale(IRequestCycle cycle, int roundIndex) {
        setAddRoundDescriptionIndex(roundIndex);
        cycle.activate(getPage());
    }
    
    public void cancelAddLocale(IRequestCycle cycle) {
        setAddRoundDescriptionIndex(-1);
        cycle.activate(getPage());
    }
    
    public boolean isLimitedSubmissions() {
        int minSubs = getRound().getMinimumSubmissions();
        if (minSubs > 100)
            return false;
        else
            return true;
    }
    
    public String getFormattedOpen() {
        DateFormat df = new SimpleDateFormat(getCourse().getDateformat(getPage().getLocale().getLanguage()));
        DateFormat df2 = new SimpleDateFormat("Z");
        Calendar cal = new GregorianCalendar();
        cal.setTime(getRound().getOpen());
        return df.format(cal.getTime()) + " GMT"+ df2.format(cal.getTime());
    }

    public String getFormattedDeadline() {
        DateFormat df = new SimpleDateFormat(getCourse().getDateformat(getPage().getLocale().getLanguage()));
        DateFormat df2 = new SimpleDateFormat("Z");
        Calendar cal = new GregorianCalendar();
        cal.setTime(getRound().getDeadLine());
        return df.format(cal.getTime()) + " GMT"+ df2.format(cal.getTime());
    }
    
    public void addMCE(IRequestCycle cycle, int round) {
        MonitorSession session = getMonitorSession();
        session.setCourse(getCourse());
        session.setRound((Round)getCourse().getRoundList().get(round - 1));
        cycle.activate("MonitorAddMultipleChoiceExercise");
    }
    
}
