// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.components;

import java.security.NoSuchAlgorithmException;

import org.apache.tapestry.BaseComponent;

import cexer.Course;
import cexer.DataStorage;
import cexer.Exercise;
import cexer.Round;
import cexer.Student;
import cexer.StudentSession;
import cexer.util.FingerprintHelper;
import cexer.util.Logging;

public abstract class PeerwiseKeyExercise extends BaseComponent {

    public abstract Course getCourse();

    public abstract Round getRound();

    public abstract Exercise getExercise();

    public abstract Student getStudent();

    public abstract StudentSession getStudentSession();

    public String getExerciseNumber() {
        int round = getRound().getRoundNumber();
        int exercise = getExercise().getNumber();
        return round + "." + exercise;
    }

    /**
     * Gets the course id.
     * @return
     */
    public int getCourseId() {
        if(getStudentSession().getUsingMonitorUI()) {
            return DataStorage.DUMMY_COURSE;
        } else {
            return getCourse().getId();
        }
    }

    public String getFingerprint() {
        Exercise e = getExercise();

        String fingerprint = FingerprintHelper.getFingerprint("" + getCourse().getId(),
                            getRound().getRoundNumber(),
                            e.getNumber(), getStudent().getId(),
                            e.getMax(), "");

        try {
            return FingerprintHelper.calculateMD5(fingerprint);
        } catch (NoSuchAlgorithmException exp) {
            exp.printStackTrace();
            Logging.error(this, "Exception calculating checksum", exp);
            return fingerprint;
        }
    }

    public String getLocalizedExerciseName() {
        return getExercise().getLocalizedName(getPage().getLocale().getLanguage());
    }
    
    /**
     * Get the student's unique key for PeerWise
     * @return the PeerWise key and null if key is not found
     */
    public String getPeerwiseKey(){
    	DataStorage ds = DataStorage.getInstance();
    	String key = ds.getPeerwiseKeyByStudentNumber(getStudent().getStudentNumber());
    	return key;
    }
}
