// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.components;

import org.apache.tapestry.IRequestCycle;
import cexer.*;
import cexer.util.TimeZoneSelectionModel;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.form.IPropertySelectionModel;

import java.util.List;
import java.util.LinkedList;
import java.util.Date;
import java.util.TimeZone;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;

/**
 * Component used for setting personal deadlines for students in the monitor interface.
 * 
 * @author cefallen
 *
 */
public abstract class SetPersonalDeadline extends BaseComponent {

    private List dayList;
    private List monthList;
    private List yearList;
    private List hourList;
    private List minuteList;
    private Calendar oldDate;
    
    /**
     * Gets the monitor session state object.
     * 
     * @return monitor session
     */
    public abstract MonitorSession getMonitorSession();

    public abstract void setRound(Round r);
    
    public abstract Round getRound();
    
    private Calendar getOldDate() {
        if(getRound()==null)
            setRound(getMonitorSession().getRound());
        else getMonitorSession().setRound(getRound());
	Date rdl = getRound().getDeadLine();
	Date pdl = getMonitorSession().getMonitoredStudent().getDeadLine(getRound().getId());
	oldDate = new GregorianCalendar();
	if(pdl == null || rdl.after(pdl)) {
	    oldDate.setTime(rdl);
	} else {
	    oldDate.setTime(pdl);
	}
	return oldDate;
    }

    public List getYearList() {
	    int year = getOldDate().get(Calendar.YEAR);
	    yearList = new LinkedList();
	    for(int i = year; i < year + 2; i++) {
		yearList.add(new Option((year == i) ? true:false, new Integer(i)));		
	}
	return yearList;
    }

    public void setYearList(List l) {}

    public List getMonthList() {
	    int month = getOldDate().get(Calendar.MONTH);
	    monthList = new LinkedList();
	    for(int i = 1; i < 13;i++) {
		Option o = new Option(false,new Integer(i));
		if(i == month+1) o.setSelected(true);
		monthList.add(o);
	}
	return monthList;
    }

    public void setMonthList(List l) {}

    public List getDayList() {
	    int day = getOldDate().get(Calendar.DATE);
	    dayList = new LinkedList();
	    for(int i = 1; i < 32;i++) {
		Option o = new Option(false, new Integer(i));
		if(day == i) o.setSelected(true);
		dayList.add(o);
	}
	return dayList;
    }

    public void setDayList(List l) {}

    public List getHourList() {
	    int hour = getOldDate().get(Calendar.HOUR_OF_DAY);
	    hourList = new LinkedList();
	    for(int i = 0; i < 24;i++) {
		Option o = new Option(false, new Integer(i));
		if(hour == i) o.setSelected(true);
		hourList.add(o);
	}
	return hourList;
    }
	
    public void setHourList(List l) {}

    public List getMinuteList() {
	    int minute = getOldDate().get(Calendar.MINUTE);
	    minuteList = new LinkedList();
	    for(int i = 0; i < 60;i++) {
		Option o = new Option(false, new Integer(i));
		if(minute == i) o.setSelected(true);
		minuteList.add(o);
	}
	return minuteList;
    }
    
/*    public String getTimezone() {
        DateFormat df = new SimpleDateFormat("Z");
        df.setTimeZone(getCourse().)
        return "GMT" + df.format(getMonitorSession().getRound().getDeadLine());
    }*/

    public IPropertySelectionModel getZoneModel() {
        return new TimeZoneSelectionModel();
    }
    
    
    public void setMinuteList(List l) {}

    public abstract Option getCurrentDay();
    public abstract void setCurrentDay(Option o);

    public abstract Option getCurrentMonth();
    public abstract void setCurrentMonth(Option o);

    public abstract Option getCurrentYear();
    public abstract void setCurrentYear(Option o);

    public abstract Option getCurrentMinute();
    public abstract void setCurrentMinute(Option o);

    public abstract Option getCurrentHour();
    public abstract void setCurrentHour(Option o);

    public abstract void setWarning(boolean state);
    
    public abstract void setWarningText(String text);
    
    public abstract String getZone();
    
    /**
     * Saves the specified (changed) deadline.
     * 
     * @param cycle
     */
    public void changeDeadline(IRequestCycle cycle) {
        MonitorSession session = getMonitorSession();
	//System.out.println(currentHour.getLabel() + ":" + currentMinute.getLabel());
	//System.out.println(currentDay.getLabel()+"."+currentMonth.getLabel()+"."+currentYear.getLabel());
	for(int i = 0; i < dayList.size(); i++) {
	    Option o = (Option)dayList.get(i);
	    if(o.getSelected()) {
		setCurrentDay(o);
		break;
	    }
	}
	for(int i = 0; i < monthList.size(); i++) {
	    Option o = (Option)monthList.get(i);
	    if(o.getSelected()) {
		setCurrentMonth(o);
		break;
	    }
	}
	for(int i = 0; i < yearList.size(); i++) {
	    Option o = (Option)yearList.get(i);
	    if(o.getSelected()) {
		setCurrentYear(o);
		break;
	    }
	}
	for(int i = 0; i < hourList.size(); i++) {
	    Option o = (Option)hourList.get(i);
	    if(o.getSelected()) {
		setCurrentHour(o);
		break;
	    }
	}
	for(int i = 0; i < minuteList.size(); i++) {
	    Option o = (Option)minuteList.get(i);
	    if(o.getSelected()) {
		setCurrentMinute(o);
		break;
	    }
	}
    Calendar cal = new GregorianCalendar();
    cal.setTimeZone(TimeZone.getTimeZone(getZone()));
    cal.set(getCurrentYear().getLabel().intValue(), getCurrentMonth().getLabel().intValue() - 1, getCurrentDay().getLabel().intValue(), getCurrentHour().getLabel().intValue(), getCurrentMinute().getLabel().intValue(), 0);
    
    SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    SimpleDateFormat df2 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    df.setTimeZone(TimeZone.getTimeZone(getZone()));
	df.setLenient(false);
    	
    Date d = new Date();
    try {
        d = df.parse(df2.format(cal.getTime()));
    } catch (ParseException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
    }
    df2.setTimeZone(TimeZone.getDefault());
    DataStorage ds = DataStorage.getInstance();
	if(ds.addPersonalDeadline(session.getMonitoredStudent(), 
				  getRound().getId(),
				  df2.format(d),
				  session.getUserId())) {
	    //System.out.println(currentHour.getLabel() + ":" + currentMinute.getLabel());
	    //System.out.println(currentDay.getLabel()+"."+currentMonth.getLabel()+"."+currentYear.getLabel());
/*        Date newDate = new Date();
            try {
                SimpleDateFormat df2 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                //df2.setTimeZone(TimeZone.getDefault());
                newDate = df2.parse(df.format(cal.getTime()));
            } catch (ParseException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }*/
        session.getMonitoredStudent().addDeadLine(getRound().getId(), d);
        setWarning(false);
        setWarningText(null);
        detach();
	    cycle.activate(getPage());
	    return;
	} else {
	    setWarning(true);
	    setWarningText("Error: Could not update deadline. Contact system administration if the problem persists.");
        detach();
	    cycle.activate(getPage());
	    return;
	}
    }

    public void detach() {
	oldDate = null;
	dayList = monthList = yearList = hourList = minuteList = null;
    }

    public String getDefaultZone() {
        return TimeZone.getDefault().getID();
    }
}
