// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.valid.ValidationDelegate;
import cexer.Student;
import cexer.StudentSession;
import cexer.DataStorage;
import cexer.util.Logging;
import cexer.util.MailerService;
import cexer.util.PasswordService;

//for email sending
import java.util.Properties;
import javax.mail.*;
import javax.mail.internet.*;


public abstract class ForgotPasswordPage extends SecurePage {

    /*
     * Tapesty 4 application state object. Fetches the injected student-session
     * object
     */
    public abstract StudentSession getStudentSession();

    public abstract void setWarning(boolean state);

    public abstract void setWarningText(String warningText);

    public abstract String getUserId();
    
    public void formSubmit(IRequestCycle cycle) {
        setWarning(false);

        ValidationDelegate delegate = (ValidationDelegate)getBeans().getBean("delegate");
        if (delegate.getHasErrors()) {
            cycle.activate(this);
            return;
        }

        DataStorage ds = DataStorage.getInstance();
        String studentNumber = getUserId().toUpperCase();

        if (ds.getStudent(studentNumber) == null) {
            setWarning(true);
            setWarningText("Unknown studentnumber.");
            cycle.activate(this);
            return;
        }

        Student s = ds.getStudent(studentNumber);

        PasswordService passwordService = PasswordService.getInstance();
        String seed = passwordService.generatePassword(32);
        ds.removeRecoveryPageData(s.getId());
        ds.setRecoveryPageData(passwordService.encrypt(seed), s.getId());

        StringBuffer traklaMsg = new StringBuffer();
        traklaMsg.append("You have requested your log-in information for TRAKLA2.\n");
        traklaMsg.append("\nFollow this link to generate a new password: https://" 
                + System.getProperty("https.server") + "/app?service=page&page=RecoverPassword&id=" 
                + seed + "\n");

        MailerService ms = MailerService.getInstance();
        boolean success = ms.mailToUsers(s.getEmail(), traklaMsg.toString());
        
        setWarning(true);
        if(!success) {            
            setWarningText("Error sending email.");
        } else {                
            setWarningText("Password sent.");
        }
        cycle.activate(this);
    }

    public void detach() {
        super.detach();
    }
}
