// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import java.util.Locale;

import org.apache.hivemind.Messages;
import org.apache.tapestry.IRequestCycle;

import cexer.*;
import cexer.util.Checker;
import cexer.util.Logging;

public abstract class LoginPage extends SecurePage {

    public abstract boolean isWarning();

    public abstract void setWarning(boolean b);

    public abstract String getWarningText();

    public abstract void setWarningText(String s);

    public abstract String getUsername();

    public abstract void setUsername(String value);

    public abstract String getPassword();

    public abstract void setPassword(String value);

    /**
     * Fetches the injected student-session object (see Login.page).
     */
    public abstract StudentSession getStudentSession();


    /**
     * @param cycle
     */
    public void formSubmit(IRequestCycle cycle) {
        if (getUsername() == null) {
            return;
        }

        String studentNumber = getUsername().toUpperCase();
        DataStorage ds = DataStorage.getInstance();
        Student student = ds.getStudent(studentNumber);
        
        // Check password
        if (student == null || student.getPassword() == null || !student.matchPassword(getPassword())) {
            setWarning(true);
            Messages messages = getMessages();
            setWarningText(messages.getMessage("error-message"));
            cexer.util.Logging.info(this, "Student " + studentNumber + " tried to log in with an invalid password.");
            return;
        }

        // Check that the user is registered
        if (!ds.hasRegistrations(studentNumber) || student.getAllCourses().size() == 0) {
            // If the course has been given as a parameter, set it to visit
            // before going to registration page
            /*
            if (courseId != -1) {
                Course c = DataStorage.getInstance().getCourse(courseId);
                if (c != null) {
                    StudentSession session = getStudentSession();
                    session.clearFields();
                    session.setCourse(c);
                }
            }
            */

            cexer.util.Logging.info(this, studentNumber + " tried to log in but has no registrations.");

            setWarning(true);
            Messages messages = getMessages();
            setWarningText(messages.getMessage("no-courses-error"));
            // cycle.activate("Register");
            return;
        }

        
        // Login
        StudentSession session = getStudentSession();
        session.clearFields();
        //session.setWarning(false);
        session.setIsLoggedIn(true);
        session.setAuthMethod(StudentSession.PASSWORD);
        session.setUserId(studentNumber);
        cycle.getEngine().setLocale(new Locale(student.getLanguage()));
        
        // Log
        cexer.util.Logging.info(this, "Logged in " + studentNumber + " (traditional)");
        ds.addLogEntry(DataStorage.LOGIN_TRADITIONAL, studentNumber);
        
        cycle.activate("Main");
    }

    public String getMessageText() {
        String msg = DataStorage.getInstance().getMessage(getPage().getLocale().getLanguage());
        return msg;
    }
    
    /**
     * Creates a link that will redirect user to the login page of the specified university.
     * @param organization must match a SessionInitiator configured in /etc/shibboleth.xml
     */
    public String makeShibLink(String organization) {
        String shibUrl = "/Shibboleth.sso/"
                + organization
                + "?target=https://"
                + System.getProperty("https.server")
                + "/app%3Fservice=page%26page=LoginShib";

        return shibUrl;
    }
    
    /**
     * Listener for the Test registration button. Redirects to register page.
     */
    public void testAccount(IRequestCycle cycle) {
        RegisterPage registerPage = (RegisterPage)cycle.getPage("Register");
        registerPage.setCourseParam(100);
        cycle.activate(registerPage);
    }
}
