// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.LinkedList;
import java.util.ListIterator;

import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.form.IPropertySelectionModel;

import cexer.*;
import cexer.util.CourseSelectionModel;
import cexer.util.Trakla2SessionListener;

/**
 * Main page of the student interface of TRAKLA2
 */
public abstract class MainPage extends SecurePage {
    private Course course;
    private Round round;
    private Exercise ex;

    public String getTitle() {
        return getMessage("title") + getStudentSession().getUserId();
    }

    public void setCourse(Course c) {
        // must set the visit's course here, since it is needed to
        // remember the user's selection of the course
        getStudentSession().setCourse(c);
        //((Visit)getVisit()).setCourse(c); 
        course = c;
    }

    public Course getCourse() {
        // if the course is not set, first try visit's course and
        // then take the first course of user's registrations
        //Visit v = (Visit)getVisit();
        StudentSession session = getStudentSession();
        if (course == null) {
            course = session.getCourse();
        }

        if (course == null) {
            course = (Course)session.getStudent().getCourseList().get(0);
            setCourse(course);
        }
        return course;
    }

    public void setRound(Round r) {
        round = r;
    }

    public Round getRound() {
        return round;
    }

    public void setExercise(Exercise e) {
        ex = e;
    }

    public Exercise getExercise() {
        return ex;
    }

    public abstract StudentSession getStudentSession();


    // XXX this is a horrible hack that should be fixed
    public int getStudentRoundPoints() {
        Student s = getStudentSession().getStudent();
        Round r = getRound();
        return s.getRoundResult(getCourse(), r.getRoundNumber());
    }

    public int getStudentCoursePoints() {
        Course c = getCourse();
        ListIterator rounds = c.getRoundList().listIterator();
        int totalPoints = 0;
        while (rounds.hasNext()) {
            Round r = (Round) rounds.next();
            setRound(r);
            totalPoints += getStudentRoundPoints();
        }
        return totalPoints;
    }

    public List getRoundInList() {
        LinkedList l = new LinkedList();
        l.add(getRound());
        return l;
    }

    public void detach() {
        super.detach();
        course = null;
        round = null;
        ex = null;
    }

    public String getFullCourseName() {
        String temp = getCourse().getCourseCode() + " " + getCourse().getLocalizedName(getLocale().getLanguage());
        if(getCourse().getOrganization() != null)
            temp = temp + " (" + getCourse().getOrganization().getLocalizedAcronym(getLocale().getLanguage()) + ")";
        return temp;
    }

    public String getCourseGrade() {
        return getCourse().getGrade(getStudentCoursePoints());
    }

    /*    public String getRoundStatus() {
    Visit v = (Visit)getVisit();
    if (v.getStudent().getRoundResult(getCourse(), getRound().getRoundNumber()) >= getRound().getMin())
        return "passedRound";
    else
        return "failedRound";
    }*/

    /** Returns a <code>CourseSelectionModel</code> of the users courses. */
    public CourseSelectionModel getCourses() {
        List clist = getStudentSession().getStudent().getCourseList();
        return new CourseSelectionModel(clist, getLocale().getLanguage());
    }

    /** Method that is called when the Show-button is pressed. */
    public void showCourse(IRequestCycle cycle) {
        // the course is already set by the PropertySelector, so
        // we just need to refresh the page.
        Student s = getStudentSession().getStudent();
        s.setDefaultCourse(getCourse());
        DataStorage.getInstance().setStudentDefaultCourse(s.getId(), getCourse().getId());
        setCourseStates(null);
        cycle.activate(this);
    }

    /** Returns true, if the user has many courses, false otherwise. */
    public boolean isManyCourses() {
	return (getCourses().getOptionCount() > 1);
    }

    public boolean getHasPersonalDeadline(int id) {
        StudentSession session = getStudentSession();
        return session.getStudent().hasPersonalDeadLine(id);
    }

    public abstract String getRowStyle1();
    public abstract String getRowStyle2();

    public String getRowStyle(int index) {
        if((index % 2)==1)
            return getRowStyle2();
        else return getRowStyle1();
    }

    public abstract String getCellNotSubmitted1();
    public abstract String getCellNotSubmitted2();
    public abstract String getCellSubmitted1();
    public abstract String getCellSubmitted2();
    public abstract String getCellSubmittedOk1();
    public abstract String getCellSubmittedOk2();
    public abstract String getCellLastSubmission1();
    public abstract String getCellLastSubmission2();

    public String getCellStyle(int index) {
        int points = getStudentSession().getStudent().getResult(getCourse(), getRound(), getExercise());
        int max = ex.getMax();
        int submissions = getStudentSession().getStudent().getSubmissions(getCourse(), getRound(), getExercise());
        int maxSubmissions = ex.getSubs();

        if(submissions==0)
            if((index % 2)==1)
                return getCellNotSubmitted2();
            else return getCellNotSubmitted1();
        else if(points>=max)
            if((index % 2)==1)
                return getCellSubmittedOk2();
            else return getCellSubmittedOk1();
        else if(submissions < (maxSubmissions - 1))
            if((index % 2)==1)
                return getCellSubmitted2();
            else return getCellSubmitted1();
        else if(submissions >= (maxSubmissions - 1))
            if((index % 2)==1)
                return getCellLastSubmission2();
            else return getCellLastSubmission1();
        return "error";
    }

    public boolean isLimitedSubmissions() {
        int minSubs = getRound().getMinimumSubmissions();
        if (minSubs > 100)
            return false;
        else
            return true;
    }

    public void exerciseSelectAction(IRequestCycle cycle) {
        StudentSession session = getStudentSession();
        Object[] params = cycle.getListenerParameters(); // problems caused when migrating from actionlinks to directlinks
        Round tempRound = (Round) getCourse().getRoundList().get(((Integer)params[0]).intValue()); // ugly fastfix
        Exercise tempExercise = (Exercise)tempRound.getExerciseList().get(((Integer)params[1]).intValue()); // this too
        session.setCourse(getCourse());
        session.setRound(tempRound);
        session.setExercise(tempExercise);
        cycle.activate("Exercise");
    }

    public Object[] getParameters() {
        Object[] params = new Object[2];
        List rounds = getCourse().getRoundList();
        List exercises = getRound().getExerciseList();
        int ri = rounds.indexOf(getRound());
        int ei = exercises.indexOf(getExercise());
        params[0] = new Integer(ri);
        params[1] = new Integer(ei);
        return params;
    }

    public abstract void setCourseStates(boolean[] o);
    public abstract boolean[] getCourseStates();

    public void setAsOnlyMaximizedRound(Round r) {
        if(getCourse().getRoundList().contains(r)) {
            boolean[] courseStates = new boolean[getCourse().getRoundList().size()];
            for(int i = 0; i < getCourse().getRoundList().size(); i++)
                courseStates[i] = false;
            courseStates[course.getRoundList().indexOf(r)] = true;
            setCourseStates(courseStates);
        }
    }

    public void setIsMaximized(IRequestCycle cycle, int index) {
        Object[] params = cycle.getListenerParameters();
        //int index = ((Integer)params[0]).intValue();
        boolean[] courseStates = getCourseStates();
        if(courseStates == null) {
            //System.out.println("setIsMaximized: initialising courseStates: " + courseStates);
            courseStates = new boolean[getCourse().getRoundList().size()];
            for(int i = 0; i < getCourse().getRoundList().size(); i++)
                courseStates[i] = false;
        }
        courseStates[index] = !courseStates[index];
        setCourseStates(courseStates);
        cycle.activate(this);
    }

    public boolean getIsMaximized(int index) {
        boolean[] courseStates = getCourseStates();

        if (courseStates == null) {
            //System.out.println("getIsMaximized: initialising courseStates: " + courseStates);
            courseStates = new boolean[getCourse().getRoundList().size()];
            for(int i = 0; i < getCourse().getRoundList().size(); i++)
                courseStates[i] = false;
        }

        if (index >= getCourse().getRoundList().size()) {
            return false;
        }

        return courseStates[index];
    }

    public boolean getHasRoundDescription() {
        String text = getRound().getLocalizedDescription(getPage().getLocale().getLanguage());

        if(text.equals("") || text == null) {
            return false;
        }

        return true;
    }

    public boolean getHasCourseDescription() {
        String text = getCourse().getLocalizedDescription(getPage().getLocale().getLanguage());

        if(text.equals("") || text == null) {
            return false;
        }

        return true;
    }

    public abstract void setShowCourseDescription(boolean state);

    public void hideCourseDescription(IRequestCycle cycle) {
        setShowCourseDescription(false);
        cycle.activate(this);
    }

    public void showCourseDescription(IRequestCycle cycle) {
        setShowCourseDescription(true);
        cycle.activate(this);
    }

    public boolean getIsAfterDeadline() {
        Calendar now = Calendar.getInstance();
        Calendar deadLine = Calendar.getInstance(getPage().getLocale());
        deadLine.setTime(getRound().getDeadLine());
        if (getHasPersonalDeadline(getRound().getId()))
            deadLine.setTime(getStudentSession().getStudent().getDeadLine(getRound().getId()));
            //deadLine.setTime(((Visit)getPage().getVisit()).getStudent().getDeadLine(getRound().getId()));
        if (now.after(deadLine))
            return true;
        else
            return false;
        }

    public String getPersonalDeadline() {
        DateFormat df = new SimpleDateFormat(getCourse().getDateformat(getPage().getLocale().getLanguage()), getPage().getLocale());
        df.setTimeZone(getCourse().getTimeZone());
        DateFormat df2 = new SimpleDateFormat("Z");
        df2.setTimeZone(getCourse().getTimeZone());
        
        Date deadline = getStudentSession().getStudent().getDeadLine(getRound().getId());
        return df.format(deadline) + " GMT" + df2.format(deadline);
    }

    public String getDeadlineFormatting() {
        if(getIsAfterDeadline())
            return "smallWarning";
        else return "";
    }

    public int getRoundSubmissions() {
        return getStudentSession().getStudent().getSubmissions(getCourse(), getRound());
    }
    
    public String getFormattedOpen() {
        DateFormat df = new SimpleDateFormat(getCourse().getDateformat(getPage().getLocale().getLanguage()), getPage().getLocale());
        df.setTimeZone(getCourse().getTimeZone());
        DateFormat df2 = new SimpleDateFormat("Z");
        df2.setTimeZone(getCourse().getTimeZone());
        Calendar cal = new GregorianCalendar();
        cal.setTime(getRound().getOpen());
        return df.format(cal.getTime()) + " GMT"+ df2.format(cal.getTime());
    }

    public String getFormattedDeadline() {
        DateFormat df = new SimpleDateFormat(getCourse().getDateformat(getPage().getLocale().getLanguage()), getPage().getLocale());
        df.setTimeZone(getCourse().getTimeZone());
        DateFormat df2 = new SimpleDateFormat("Z");
        df2.setTimeZone(getCourse().getTimeZone());
        Calendar cal = new GregorianCalendar();
        cal.setTime(getRound().getDeadLine());
        return df.format(cal.getTime()) + " GMT"+ df2.format(cal.getTime());
    }
}

