/**
 *
 */
package cexer.pages;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.RedirectException;
import org.apache.tapestry.engine.ILink;

import cexer.Choice;
import cexer.ChoiceGroup;
import cexer.DataStorage;
import cexer.Exercise;
import cexer.MonitorSession;
import cexer.MultipleChoiceExerciseTemplate;
import cexer.Question;
import cexer.QuestionGroup;
import cexer.Round;
import cexer.util.mceparser.MceParser;

/**
 * Test page, used for adding a multiple choice exercise to the database.
 *
 * @author cefallen
 *
 */
public abstract class MonitorAddMultipleChoiceExercisePage extends SecurePage {
    
    public static final int ADD_TO_COURSE = 0;
    
    public static final int ADD_WITHOUT_COURSE = 1;    

    public abstract MonitorSession getMonitorSession();

    public abstract String getExerciseName();
    
    public abstract String getExerciseDescription();
    
    public abstract int getSelectedOption();
    
    public abstract void setSelectedOption(int optionNumber);
    
    public abstract int getMaxPoints();
    
    public abstract void setWarningText(String message);
    
    public abstract void setWarning(boolean state);
    
    public abstract String getExerciseXml();
    
    public abstract void setExerciseXml(String value);    
    
    public abstract boolean isUnlimitedSubmissions();
    
    public abstract int getMaxSubmissions();

    public void addMcExercise(IRequestCycle cycle) {
        if(getExerciseName() == null || getExerciseName().equals("")) {
            setWarning(true);
            setWarningText("Error: exercise name is empty.");
            return;
        }
        if(getExerciseDescription() == null || getExerciseDescription().equals("")) {
            setWarning(true);
            setWarningText("Error: exercise description is empty.");
            return;
        }
        if(getExerciseXml() == null || getExerciseXml().equals("")) {
            setWarning(true);
            setWarningText("Error: exercise xml is empty.");
            return;
        }
        if(getMaxPoints() < 0 && getSelectedOption() == 0) {
            setWarning(true);
            setWarningText("Error: negative points not allowed.");
            return;
        }
        if(getMaxSubmissions() == 0 && !isUnlimitedSubmissions() && getSelectedOption() == 0) {
            setWarning(true);
            setWarningText("Error: zero submissions not allowed.");
            return;            
        }
        setWarning(false);
        List groups = new LinkedList();
        try {
            MceParser parser = new MceParser();
            parser.parse(getExerciseXml());
            //System.out.println(((QuestionGroup) parser.getQuestionGroups().get(0)).toXml());
            groups = parser.getQuestionGroups();
        } catch (Exception e) {
            // TODO: handle exception
            e.printStackTrace();
            setWarning(true);
            setWarningText("Error parsing the xml:" + e.getMessage());
            return;
        }
        HashMap names = new HashMap();
        HashMap descriptions = new HashMap();


        DataStorage ds = DataStorage.getInstance();

        System.out.println("Adding McExercise...");
        Round round = null;
        if(getSelectedOption() == 0)
            round = getMonitorSession().getRound();;
        names.put("en", getExerciseName());
        descriptions.put("en", getExerciseDescription());
        int submissions = getMaxSubmissions();
        if(isUnlimitedSubmissions())
            submissions = -1;
        int res = ds.addMultipleChoiceExercise(groups, descriptions, names, round, getNumberOfExerciseToAdd(), getMaxPoints(), submissions);
        System.out.println("Result = " + res);
        setWarning(true);
        if(res > 0) {
            setWarningText("Exercise added with id " + res + ".");
            setExerciseXml("");
            if(getSelectedOption() == 0) {
                String[] params = new String[2];
                params[0] = getExerciseName();
                params[1] = getExerciseDescription();
                MultipleChoiceExerciseTemplate et = new MultipleChoiceExerciseTemplate(params, new LinkedList());
                et.setId(res);
                et.addLocalizedData("en", getExerciseName(), getExerciseDescription());
                et.setQuestionGroups(groups);
                submissions = getMaxSubmissions();
                if(isUnlimitedSubmissions())
                    submissions = 35000;
                Exercise e = new Exercise(et, getMaxPoints(), getNumberOfExerciseToAdd(), submissions);
                ((Round)getMonitorSession().getCourse().getRoundList().get(getMonitorSession().getRound().getRoundNumber() - 1)).addExercise(e);
                getMonitorSession().setRound((Round)getMonitorSession().getCourse().getRoundList().get(getMonitorSession().getRound().getRoundNumber() - 1)); // to ensure that the new exercise is added to all persistent data (round) fields.
                getMonitorSession().clearCourseData();
            }
        }
        else setWarningText("Error: Exercise not added.");        
    }

    public boolean isHasAllParameters() {        
        return (getMonitorSession().getCourse() != null && getMonitorSession().getRound() != null && getMonitorSession().isAdmin());
    }
    
    public int getNumberOfExerciseToAdd() {
        return getMonitorSession().getRound().getExerciseList().size() + 1;
    }
    
    public void cancel(IRequestCycle cycle) {
        cycle.activate("MonitorMain");
    }

}
