// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import org.apache.tapestry.IRequestCycle;
import cexer.*;
import cexer.util.Logging;

public abstract class MonitorEditHelpPage extends SecurePage {

    public abstract boolean getHasMessage();
    public abstract void setHasMessage(boolean hasMessage);
    
    public abstract String getMessage();
    public abstract void setMessage(String message);
    
    
    // Returns the working copy of the help content
    public abstract String getModifiedContent();
    
    public abstract void setModifiedContent(String content);

    
    public abstract String getTheLocale();
    public abstract void setTheLocale(String s);

    public abstract int getCourseId();
    public abstract void setCourseId(int id);
    
    public Course getCourse() {
        DataStorage ds = DataStorage.getInstance();
        return ds.getCourse(getCourseId());
    }
    
    public abstract MonitorSession getMonitorSession();
    
    /**
     * Returns the working copy of the content. It is fetched from the
     * database on the first time and from a persistent property from
     * there on.
     * @return help content
     */
    public String getContent() {
        String modifiedContent = getModifiedContent();
        if (modifiedContent != null) {
            // Return the working copy
            return modifiedContent;
        } else {
            // Fetch from database (on the first load)
            DataStorage ds = DataStorage.getInstance();
            String help = ds.getHelp(getCourseId(), getTheLocale());
            
            // Save it as a local copy to reduce database queries 
            setModifiedContent(help);
            
            return help;
        }
    }
    
    public void setContent(String content) {
        setModifiedContent(content);
    }

    
    public void save(IRequestCycle cycle) {
        // In case the user erases text 
        if (getModifiedContent() == null) {
            setModifiedContent("");
        }
        
        DataStorage ds = DataStorage.getInstance();
        ds.setHelp(getCourseId(), getTheLocale(), getContent());
        
        setHasMessage(true);
        setMessage("Page succesfully saved");
    }
    
    public void preview(IRequestCycle cycle) {
        // In case the user erases text
        if (getModifiedContent() == null) {
            setModifiedContent("");
        }
    }

    public void toCourse(IRequestCycle cycle) {
        cycle.activate("MonitorMain");
    }

    public String getName() {
        Course course = getCourse();
        String locale = getTheLocale();
        
        if (course == null || locale == null) {
            return "";
        }
                
        return course.getLocalizedName(locale);
    }
    
}
