// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageBeginRenderListener;
import java.util.List;
import java.util.LinkedList;
import java.util.HashMap;
import java.util.Locale;

import cexer.*;
import cexer.util.Logging;

public abstract class MonitorLocalizeAppletPage extends SecurePage implements PageBeginRenderListener {

    public abstract void setCourseId(int id);
    public abstract int getCourseId();
    
    public abstract void setExerciseId(int id);
    public abstract int getExerciseId();
    
    public abstract void setLanguage(String lang);
    public abstract String getLanguage();
    
    public abstract void setRow(String title);
    public abstract String getRow();
    public abstract void setColumn(String column);
    public abstract String getColumn();
    
    public abstract void setRowIndex(int index);
    public abstract int getRowIndex();
    public abstract void setColumnIndex(int index);
    public abstract int getColumnIndex();
    
    public abstract void setStringArray(List list);
    public abstract List getStringArray();
    
    public abstract void setNewRowTitle(String title);
    public abstract String getNewRowTitle();
    
    /**
     * Gets the monitor interface user session object
     * @return the session object
     */
    public abstract MonitorSession getMonitorSession();
    
    public Course getCourse() {
        int courseId = getCourseId();
        
        if (courseId > 0) {
            DataStorage ds = DataStorage.getInstance();
            return ds.getCourse(getCourseId());
        } else {
            return null;
        }
    }
    
    public Exercise getExercise() {
        // XXX: This is stupid. Exercise id should be given as a parameter
        // to this page, just like the course id. However, this cannot be
        // done as there is no getExercise() in DataStorage.
        
        return getMonitorSession().getExercise();
    }
    
    public boolean isCourseSpecific() {
        if (getCourseId() > 0) {
            return true;
        } else {
            return false;
        }
    }
    
    public void pageBeginRender(PageEvent event) { 
        DataStorage ds = DataStorage.getInstance();
        setStringArray(ds.getAppletStringArray(getExerciseId(),
                getLanguage(),
                getCourseId()));
    }
    
    public List getRows() {
        List list1 = getStringArray();
        List list2 = new LinkedList();
        
        // Get the leftmost column of the matrix
        for (int i=0; i<list1.size(); i++) {
            list2.add(((String[])list1.get(i))[0]);
        }
        
        list2.add(null);
        
        return list2;
    }
    
    public List getColumns() {
        List list = new LinkedList();
        list.add("en");
        list.add("fi");
        
        if (isCourseSpecific()) {
            list.add("en");
            list.add("fi)");
        }
        
        return list;
    }
    
    public String getRowTitle() {
        List list = getStringArray();
        
        int row = getRowIndex();
        
        if (row < list.size()) {
            String[] array = (String[])list.get(row);
            
            return array[0];
        } else {
            return null;
        }
    }

    /**
     * Returns true if user is allowed to edit current cell.
     * @return
     */
    public boolean isPermitEditCell() {
        // Disable foreign language when editing in english
        if (getLanguage().equals("en") && getColumnIndex() % 2 == 1) {
            return false;
        }
        
        // If user is not admin, allow only course specific changes
        if (!getMonitorSession().isAdmin() && getColumnIndex() < 2) {
            return false;
        }
        
        return true;
    }
    
    public String getCell() {
        List list = getStringArray();
        
        int row = getRowIndex();
        int column = getColumnIndex();
        
        if (row < list.size() && column < 5) {
            String[] array = (String[])list.get(row);
            
            return array[column + 1];
        } else {
            return null;
        }
    }
    
    public void setCell(String userValue) {
        DataStorage ds = DataStorage.getInstance();
        List list = getStringArray();
        int row = getRowIndex();
        int column = getColumnIndex();
        
        String[] array;
        String existingValue;
        if (row < list.size()) {
            array = (String[])list.get(row);
            existingValue = array[column + 1];
        } else {
            array = null;
            existingValue = null;
        }
        
        // Is there anything to do?
        if (existingValue == null && userValue == null
                || (existingValue != null && existingValue.equals(userValue))) {
            return;
        }

        // Property name
        String propertyName;
        if (row >= list.size()) {
            propertyName = getNewRowTitle();
        } else {
            propertyName = array[0];
        }
        
        if (propertyName == null || propertyName.equals("<new>")) {
            return;
        }
        
        
        // cId
        int cId;
        if (column < 2) {
            // Global
            cId = 0;
        } else {
            // Course specific
            cId = getCourseId();
        }
        
        // Language
        String lang;
        if (column == 0 || column == 2) {
            lang = "en";
        } else {
            lang = getLanguage();
        }
        
        // eId
        int eId = getExerciseId();
        
        // Perform query
        ds.updateAppletString(cId, lang, eId, propertyName, userValue, existingValue);
    }
    
    /**
     * Listener for Save botton.
     */
    public void save() {
        // Reset new property field for usability
        
        if (getNewRowTitle() != null && getNewRowTitle().equals("<new>")) {
            setNewRowTitle("<new>");
        } else {
            setNewRowTitle("");
        }
    }
    
    public abstract void setSelectedPreview(String column);
    public abstract String getSelectedPreview();
    
    /**
     * @return -1 = none, 0 = global en, ...
     */
    public int getPreviewColumn() {
        String columnName = getSelectedPreview();
        
        if (columnName == null) {
            return -1;
        } else {
            return Integer.valueOf(columnName).intValue();
        }
    }
    
    public boolean isShowPreview() {
        return getPreviewColumn() >= 0; 
    }
    
    public Course getPreviewCourse() {
        if (getPreviewColumn() < 2) {
            return DataStorage.getInstance().getCourse(DataStorage.DUMMY_COURSE);
        } else {
            return getCourse();
        }
    }
    
    public String getPreviewLang() {
        if (getPreviewColumn() % 2 == 0) {
            return "en";
        } else {
            return getLanguage();
        }
    }
    
    
    public void editGlobals(IRequestCycle cycle) {
        MonitorLocalizeAppletPage nextPage = (MonitorLocalizeAppletPage)cycle.getPage("MonitorLocalizeApplet");
        
        nextPage.setExerciseId(0);
        nextPage.setCourseId(getCourseId());
        nextPage.setLanguage(getLanguage());
        
        cycle.activate(nextPage);
    }
    
    public void editLocals(IRequestCycle cycle) {
        MonitorLocalizeAppletPage nextPage = (MonitorLocalizeAppletPage)cycle.getPage("MonitorLocalizeApplet");
        
        nextPage.setExerciseId(getMonitorSession().getExercise().getExerciseTemplate().getId());
        nextPage.setCourseId(getCourseId());
        nextPage.setLanguage(getLanguage());
        
        cycle.activate(nextPage);
    }
}
