// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;


import org.apache.tapestry.IRequestCycle;

import cexer.*;

import java.util.List;
import java.util.ListIterator;


public abstract class MonitorMainPage extends SecurePage {

    public abstract MonitorSession getMonitorSession();
    
    private String student; // is required, used in finding student, not persistent

    // charts
    public static String POINT_DISTRIBUTION = "points";
    public static String GRADE_DISTRIBUTION = "grades";
    public static String SERVER_ACTIVITY = "activity";

    // page state
    public static final int DETAILS = 0;
    public static final int STATISTICS = 1;
    public static final int STUDENTS = 2;
    public static final int ROUND_POINT_DISTRIBUTION = 3;

    public abstract void setIndex(int i);

    public abstract int getIndex();

    public Course getCourse() {
  		MonitorSession session = getMonitorSession();
		Course c = session.getCourse();
  		if(c.getId() == DataStorage.DUMMY_COURSE) {
		    session.setCourse((Course)session.getStudent().getCourseList().get(0));
		    return session.getCourse();
		}
		return c;
    }

    public void setCourse(Course c) {
    }


    /**
     * @return true if the current view mode is course details
     */
    public boolean isShowDetails() {
        return getTabShowed() == DETAILS;
    }

    /**
     * @return true if the current view mode is round statistics 
     */
    public boolean isShowStatistics() {
        return getTabShowed() == STATISTICS;
    }
    
    /**
     * @return true if the current view mode is course students
     */
    public boolean isShowStudents() {
        return getTabShowed() == STUDENTS;
    }
    
    /**
     * @return true if the current view mode is round point distribution
     */
    public boolean isShowPointDistribution() {
        return getTabShowed() == ROUND_POINT_DISTRIBUTION;
    }

    /**
     * Gets the content of the student number textbox in student search
     * @return
     */
    public abstract String getStudentToFind();

    public abstract void setStudentToFind(String s);
    
    public abstract String getStudentNameToFind();
    
    public abstract void setStudentNameToFind(String name);
    
    public abstract String getStudentIdToFind();
    
    public abstract void setStudentIdToFind(String name);

    
    public abstract void setActivity(boolean state);
    
    public abstract boolean isActivity();

    public abstract MonitorSearchResultsPage getResultsPage();
    
    abstract public int getTabShowed();
    
    abstract public void setTabShowed(int tabShowed);

    public abstract void setWarning(boolean state);
    
    public abstract void setSearchStringTooShort(boolean state);
    
    public abstract void setStudentNotFound(boolean state);

    public void detach() {
		super.detach();
		student = null;
	    setWarning(false);
    }
    
    /**
     * Clear the system cache. This removes cached courses, students and exercises.
     * @param cycle
     */
    public void clearCache(IRequestCycle cycle) {
		DataStorage ds = DataStorage.getInstance();
		ds.emptyCache();
    }

    /**
     * Link to the all exercises page
     * @param cycle
     */
    public void allExercises(IRequestCycle cycle) {
        MonitorSession session = getMonitorSession();
        DataStorage ds = DataStorage.getInstance();
        session.setRound(ds.getAllExercises());
        //v.getRound().setAdditionalAppletParameters(new LinkedList());
        session.setCourse(ds.getCourse(DataStorage.DUMMY_COURSE));
        //v.getCourse().setAdditionalAppletParameters(new LinkedList());
        cycle.activate("MonitorAllExercises");
    }


    public void showCourseDetails(IRequestCycle cycle) {
		setTabShowed(DETAILS);
    }

    public void showActivityLog(IRequestCycle cycle) {
        setActivity(!isActivity());
    }

    public void showCourseStatistics(IRequestCycle cycle) {
		setTabShowed(STATISTICS);
    }
 
    public void showCourseStudents(IRequestCycle cycle) {
		setTabShowed(STUDENTS);
    }

    public void showPointDistribution(IRequestCycle cycle) {
        setTabShowed(ROUND_POINT_DISTRIBUTION);
    }
    
    /**
     * Searches for students in the database. The search is done by name or student
     * number, depending on where the search data is entered.
     * @param cycle
     */
    public void searchStudent(IRequestCycle cycle) {
        MonitorSession session = getMonitorSession();
        if((getStudentToFind() == null || getStudentToFind().length() < 2)
           && (getStudentNameToFind() == null || getStudentNameToFind().length() < 3)
           && (getStudentIdToFind() == null || getStudentIdToFind().length() < 2))
        {
            // Error situation
            setWarning(true);
            setSearchStringTooShort(true);
            return;
        }

        DataStorage ds = DataStorage.getInstance();
        List courses = session.getStudent().getAllCourses(); //XXX is the user really instructor on all these courses??
        List students = ds.searchStudents(getStudentNameToFind(), getStudentToFind(), getStudentIdToFind(), courses);

        if(students == null || students.isEmpty()) {
            setWarning(true);
            setStudentNotFound(true);
            return;
        }

        if(students.size() == 1) {
            // Only one student found
            Student stu = (Student)students.get(0);
            if(stu != null) {
                session.setMonitoredStudent(stu);
                List studentCourses = stu.getAllCourses();
                if(!studentCourses.contains(getMonitorSession().getCourse())) {
                    Course tempCourse = null;
                    ListIterator studentCourseIterator = studentCourses.listIterator();
                    while(studentCourseIterator.hasNext()) {
                        tempCourse = (Course)studentCourseIterator.next();
                        if(courses.contains(tempCourse)) {
                            session.setCourse(tempCourse);
                            break;
                        }
                    }
                }
                cycle.activate("MonitorStudent");
                return;
            }
        } else {
            // Many students found
            MonitorSearchResultsPage resultsPage = getResultsPage();
            resultsPage.setResults(students);
            cycle.activate(resultsPage);

            return;
        }

        setWarning(true);
        setStudentNotFound(true);
        cycle.activate(this);
   }

    public void textResults(IRequestCycle cycle) {
        MonitorTextResultsPage next_page = (MonitorTextResultsPage) cycle.getPage("MonitorTextResults");
        next_page.setStudentResults(getStudentResults());

        cycle.activate(next_page);
    }

    public Statistics getStatistics() {
        return  getMonitorSession().getStatistics();
    }

    // Not used anywhere (?)
    //public List getRoundStatisticsList() {
    //    return getStatistics().getRounds();
    //}

    /**
     * @return true if the user has instructor rights on multiple courses.
     */
    public boolean isManyCourses() {
        MonitorSession session = getMonitorSession();
        List l = session.getStudent().getCourseList();
        return l.size() > 1;
    }
    
    /**
     * @return string containing the localized organization name and acronym
     */
    public String getOrganizationName() {
        return getCourse().getOrganization().getLocalizedName(getLocale().getLanguage()) + " (" + getCourse().getOrganization().getLocalizedAcronym(getLocale().getLanguage()) + ")";
    }

    /**
     * Opens a popup with a student view of the current course 
     * @param cycle
     * @return Student interface main page object
     */
    public MainPage studentMainPagePopup(IRequestCycle cycle) {
        DataStorage ds = DataStorage.getInstance();
        Student tempStudent = ds.getStudent(getMonitorSession().getStudent().getId());

        if(tempStudent == null || !tempStudent.getCourseList().contains(getCourse())) {
            //Create dummy student for course
            tempStudent = new Student("MONITOR", "Monitor user");
            tempStudent.setCourse(getCourse());
        }
        //Login with tempStudent
        MainPage nextPage = (MainPage)cycle.getPage("Main");

        StudentSession studentSession = nextPage.getStudentSession();
        studentSession.setCourse(getCourse());
        //session.setWarning(false);
        studentSession.setIsLoggedIn(true);
        studentSession.setUserId(tempStudent.getId());
        studentSession.setName(tempStudent.getName());
        studentSession.setUsingMonitorUI(true);
        //cycle.getEngine().setLocale(new Locale(tempStudent.getLanguage()));
        cexer.util.Logging.info(this, "Monitor " + getMonitorSession().getUserId() + " logged in as a student from Monitor UI");

        return nextPage;
    }
        
    public List getStudentResults() {
        return getMonitorSession().getStudentResults();
	}
    
    public String getTabClass(int tab) {
        if(tab == getTabShowed())
            return "selected";
        else return "";
    }
}
