// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import org.apache.tapestry.html.BasePage;
import org.apache.tapestry.IRequestCycle;
import cexer.*;
import cexer.util.Lock;

public abstract class MonitorModifyDescriptionPage extends SecurePage {

    private String descr;
    private String name;

    public abstract String getTheLocale();
    public abstract void setTheLocale(String s);

    public abstract void setErrorMessage(String message);
    
    public abstract MonitorSession getMonitorSession();
    
    public String getDescription() {
        //System.out.println("getDescription(): locale: " + getTheLocale());
        return getMonitorSession().getExercise().getLocalizedDescription(getTheLocale());
    }

    public void setDescription(String s) {
	descr = s;
    }
    
    public void setEditableName(String name) {
        this.name = name;
    }
    
    public String getEditableName() {
        return getName();
    }

    public void modifyDefault(IRequestCycle cycle) {
	setEditError(false);
	if(descr == null || descr.equals("")) {
        setEditError(true);
        setErrorMessage("Error: The description is empty.");
        cycle.activate(this);
        return;
	}
    if(name == null || name.equals("")) {
        setEditError(true);
        setErrorMessage("Error: Name is empty.");
        cycle.activate(this);
        return;
    }    
	DataStorage ds = DataStorage.getInstance();
        MonitorSession session = getMonitorSession();
	String locale = getTheLocale();
	    Exercise ex = session.getExercise();
	if(hasLock()) {
	    //System.out.println("MAKING MODIFICATION with locale: " + locale + " id " + session.getRound().getId() + " number " + ex.getNumber() + " template: " + ex.getExerciseTemplate().getId());
	    boolean success = ds.updateDescription(name, descr, locale,ex.getExerciseTemplate().getId(), 0);
	    if(success) {
		ExerciseTemplate et = ex.getExerciseTemplate();
		String name = et.getLocalizedName(getTheLocale());
		et.addLocalizedData(getTheLocale(),name,descr);
	    } else {
		setEditError(true);
        setErrorMessage("Error: Could not store the changes, perhaps your lock on the exercise timed out. If the problem persists, please inform the administrators.");
		cycle.activate(this);
		return;
	    }
	    releaseLock();
	    cycle.activate("MonitorTestExercise");
	    return;
	} 
	setEditError(true);
	cycle.activate(this);
    }
    
    public void modifyCourseSpecific(IRequestCycle cycle) {
        setEditError(false);
        if(descr == null || descr.equals("")) {
            setEditError(true);
            setErrorMessage("Error: The description is empty.");
            cycle.activate(this);
            return;
        }
        if(name == null || name.equals("")) {
            setEditError(true);
            setErrorMessage("Error: Name is empty.");
            cycle.activate(this);
            return;
        } 
        DataStorage ds = DataStorage.getInstance();
        MonitorSession session = getMonitorSession();
        String locale = getTheLocale();
        Exercise ex = session.getExercise();
        if(hasLock()) {
            //System.out.println("MAKING MODIFICATION with locale: " + locale + " id " + session.getRound().getId() + " number " + ex.getNumber() + " template: " + ex.getExerciseTemplate().getId());
            boolean success = ds.updateDescription(name, descr, locale,ex.getExerciseTemplate().getId(), session.getCourse().getId());
            if(success) {
                ex.addLocalizedData(getTheLocale(), name, descr);
            } else {
                setEditError(true);
                setErrorMessage("Error: Could not store the changes, perhaps your lock on the exercise timed out. If the problem persists, please inform the administrators.");                
                cycle.activate(this);
                return;
            }
            releaseLock();
            cycle.activate("MonitorTestExercise");
            return;
        } 
        setEditError(true);
        cycle.activate(this);
    }

    public void removeCourseSpecific(IRequestCycle cycle) {
        setEditError(false);
        DataStorage ds = DataStorage.getInstance();
        MonitorSession session = getMonitorSession();
        String locale = getTheLocale();
        Exercise ex = session.getExercise();
        if(hasLock()) {
            //System.out.println("MAKING MODIFICATION with locale: " + locale + " id " + session.getRound().getId() + " number " + ex.getNumber() + " template: " + ex.getExerciseTemplate().getId());
            boolean success = ds.removeDescription(locale,ex.getExerciseTemplate().getId(), session.getCourse().getId());
            if(success) {
                ex.removeLocalizedData(getTheLocale());
            } else {
                setEditError(true);
                cycle.activate(this);
                return;
            }
            releaseLock();
            cycle.activate("MonitorTestExercise");
            return;
        } 
        setEditError(true);
        cycle.activate(this);
    }
    
    public String getConfirmSaveScript() {
        MonitorSession session = getMonitorSession();
        if(session.getExercise().hasCourseSpecificDescription(getTheLocale()))
            return "return confirm('Saving the current course specific description as the new default for all courses. Are you sure?');";
            else return "return true;";
    }
    
    public String getConfirmDeleteScript() {
        return "return confirm('Deleting the current course specific description. Are you sure?');";
    }
    
    public void toMonitor(IRequestCycle cycle) {
	setEditError(false);
	releaseLock();
	cycle.activate("MonitorTestExercise");
    }

    public void toMain(IRequestCycle cycle) {
	setEditError(false);
	releaseLock();
	cycle.activate("MonitorMain");
    }

    public String getName() {
        String loc = getTheLocale();
        return getMonitorSession().getExercise().getLocalizedName(loc);
    }
    
    public boolean isCourseSpecificPossible() {
        MonitorSession session = getMonitorSession();
        if(session.getCourse().getId() == 0 || session.getCourse().getId() == DataStorage.DUMMY_COURSE)
            return false;
        else return true;
    }

    public boolean isCourseSpecificExists() {
        MonitorSession session = getMonitorSession();
        if(session.getExercise().hasCourseSpecificDescription(getTheLocale()))
            return true;
        else return false;
    }
    
    private boolean releaseLock() {
	return checkLock(true);
    }

    private boolean checkLock(boolean b) {
	DataStorage ds = DataStorage.getInstance();
    MonitorSession session = getMonitorSession();
	Integer eid = new Integer(session.getExercise().getExerciseTemplate().getId());
	synchronized(this) {
	    Lock l = ds.renewLock(eid, getMonitorSession());
	    if(l == null) 
		return false;
	    if(l.getId() == session) {
		if(b) {
		    ds.removeLock(eid);
		}
		return true;
	    }
	}
	return false;
    }

    private boolean hasLock() {
	return checkLock(false);
    }

    public void detach() {
	super.detach();
	descr = null;
    //releaseLock();
	//System.out.println("ModifyPage: detached");
    setEditError(false);
    }
    
    public boolean externalReleaseLock() {
        releaseLock();
        return true;
    }    

    public abstract boolean isEditError();
   
    public abstract void setEditError(boolean b);

}
