// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import org.apache.tapestry.html.BasePage;
import org.apache.tapestry.IRequestCycle;
import cexer.*;
import cexer.util.CourseSelectionModel;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.LinkedList;
import java.util.Iterator;
import java.util.ListIterator;

public abstract class MonitorStudentPage extends SecurePage {

    //private List courses;

    public abstract MonitorSession getMonitorSession();
    
    public abstract int getRoundIndex();
    
    public abstract int getExerciseIndex();
    
    //public abstract void setExerciseIndex(int e);
    
    //public abstract void setRoundIndex(int e);
 
    public abstract Round getRound();
    
    public abstract Exercise getExercise();    
    
    public abstract boolean isShowMove();
    public abstract void setShowMove(boolean b);

    public abstract boolean isShowEmail();
    
    public abstract void setShowEmail(boolean b);
    
    public abstract void setWarning(boolean b);
    
    public abstract int getEditingIndex();

    public abstract void setEditingIndex(int index);
    
    public abstract void setEditingExerciseIndex(int index);
    
    public abstract int getEditingExerciseIndex();
    
    public abstract String getRowStyle1();
    public abstract String getRowStyle2();
    
    
    public Object[] createParameterArray(int parameter1, int parameter2) {
        Object[] tempArray = new Object[2];
        tempArray[0] = new Integer(parameter1);
        tempArray[1] = new Integer(parameter2);
        return tempArray;
    }
    
    //public void setRound(Round value) { round = value; }
    //public Exercise getExercise() { return exercise; }
    //public void setExercise(Exercise value) { exercise = value; } 

    public int getPoints() {
        MonitorSession session = getMonitorSession();
        Course course = session.getCourse();
        return session.getMonitoredStudent().getResult(course, getRound(), getExercise());
    }

    public int getRoundPoints() {
        MonitorSession session = getMonitorSession();
        Course course = session.getCourse();
        return session.getMonitoredStudent().getRoundResult(course, getRound().getRoundNumber());
    }

    public int getCoursePoints() {
        MonitorSession session = getMonitorSession();
        Course c = session.getCourse();
        return session.getMonitoredStudent().getCourseResult(c);
    }

    public String getCourseGrade() {
        MonitorSession session = getMonitorSession();
        Course c = session.getCourse();
        return session.getMonitoredStudent().getGrade(c);
    }

    public int getSubmissions() {
        MonitorSession session = getMonitorSession();
        Course course = session.getCourse();
        return session.getMonitoredStudent().getSubmissions(course, getRound(), getExercise());

    }

    public String getExerciseName() {
        return getExercise().getLocalizedName(getPage().getLocale().getLanguage());
    }

    public void detach() {
	super.detach();
	//courses = null;
    setMoveToCourse(null);
    setCurrentCourse(null);
    }

    public boolean getHasPersonalDeadline() {
        return getMonitorSession().getMonitoredStudent().hasPersonalDeadLine(getRound().getId());
    }

    public String getPersonalDeadline() {
        Student s = getMonitorSession().getMonitoredStudent();
        DateFormat df = new SimpleDateFormat(getMonitorSession().getCourse().getDateformat(getPage().getLocale().getLanguage()), getPage().getLocale());
        df.setTimeZone(getMonitorSession().getCourse().getTimeZone());
        DateFormat df2 = new SimpleDateFormat("Z");        
        df2.setTimeZone(getMonitorSession().getCourse().getTimeZone());
        
        Date deadline = s.getDeadLine(getRound().getId());
        return df.format(deadline) + " GMT" + df2.format(deadline);
    }

    public String getFormattedDeadline(){
        Student s = getMonitorSession().getMonitoredStudent();
        DateFormat df = new SimpleDateFormat(getMonitorSession().getCourse().getDateformat(getPage().getLocale().getLanguage()), getPage().getLocale());
        df.setTimeZone(getMonitorSession().getCourse().getTimeZone());
        DateFormat df2 = new SimpleDateFormat("Z");
        df2.setTimeZone(getMonitorSession().getCourse().getTimeZone());
        Calendar cal = new GregorianCalendar();
        cal.setTime(getRound().getDeadLine());
        return df.format(cal.getTime()) + " GMT"+ df2.format(cal.getTime());
    }

    public void showEditExercise(IRequestCycle cycle) {
        MonitorSession session = getMonitorSession();
        Object[] params = cycle.getListenerParameters(); // problems caused when migrating from actionlinks to directlinks
        Round tempRound = (Round)session.getCourse().getRoundList().get(((Integer)params[0]).intValue() - 1); // ugly fastfix
        Exercise tempExercise = (Exercise)tempRound.getExerciseList().get(((Integer)params[1]).intValue() - 1); // this too
        session.setRound(tempRound); 
        session.setExercise(tempExercise);
        //cycle.activate("MonitorStudentExercise");
        setEditingExerciseIndex(((Integer)params[1]).intValue() - 1);
        cycle.activate(this);
    }
    
    public void hideEditExercise(IRequestCycle cycle) {
        setEditingExerciseIndex(-1);
        cycle.activate(this);        
    }

    public void personalDeadlineSelection(IRequestCycle cycle) {
	/* MonitorSetPersonalDeadlinePage next_page = (MonitorSetPersonalDeadlinePage) cycle.getPage("MonitorSetPersonalDeadline");
    MonitorSession session = getMonitorSession(); */
    Object[] params = cycle.getListenerParameters();
    setEditingIndex(((Integer)params[0]).intValue() - 1);
    //Round tempRound = (Round)session.getCourse().getRoundList().get(((Integer)params[0]).intValue() - 1);
    //next_page.setRound(tempRound);
	cycle.activate(this);
    }

    // returns a List containing the CourseWrappers for each course
    /*
     public List getCourseList() {
	if(courses == null) {
        MonitorSession session = getMonitorSession();
	    List l = session.getStudent().getCourseList();
	    courses = new LinkedList();
	    Iterator i = l.iterator();
	    while(i.hasNext()) {
		Course c = (Course)i.next();
		courses.add(new CourseWrapper(c));
	    }
	}
	return courses;
    }
     */
     public CourseSelectionModel getCourseList() {
        List clist = getMonitorSession().getStudent().getCourseList();
        return new CourseSelectionModel(clist,getPage().getLocale().getLanguage());
     } 
    

    public abstract void setMoveToCourse(Course c);

    public abstract void setCurrentCourse(Course c);
    
    public abstract Course getCurrentCourse();
    
    public abstract Course getMoveToCourse();

    // returns the localized name of the current course
    /*
    public String getLocalizedName() {
	Course c = getCurrentCourse();
	return c.getLocalizedName(getLocale().getLanguage());
    }*/

    public void showMoveForm(IRequestCycle cycle) {
	setShowMove(!isShowMove());
	cycle.activate(this);
    }
    
    public void showChangeEmailForm(IRequestCycle cycle) {
        setShowEmail(!isShowEmail());
        cycle.activate(this);
    }
    
    public void hideForms(IRequestCycle cycle) {
        cycle.activate(this); //The showMove-value is automaticly resetted on page rollback (it isn't persistent)
    }

    public void moveStudent(IRequestCycle cycle) {
        MonitorSession session = getMonitorSession();
	DataStorage ds = DataStorage.getInstance();
	Course from = session.getCourse();
	Course to = getMoveToCourse();
	/*
	List c = getCourseList();
	for(int i = 0; i < c.size(); i++) {
	    CourseWrapper tmp = (CourseWrapper)c.get(i);
	    if(tmp.isSelected()) {
		to = tmp.getCourse();
		break;
	    }
	}
    */
	Student s = session.getMonitoredStudent();
	boolean success = false;
	if(to != null)
	    success = ds.moveStudent(s, from, to);
	if(success) {
	    List res = session.getStudentResults();
	    res.remove(s);
        setShowMove(false);
	    cycle.activate("MonitorMain");
	} else {
	    MonitorStudentPage next_page = (MonitorStudentPage)cycle.getPage("MonitorStudent");
        setShowMove(false);
	    next_page.setWarning(true);
	    cycle.activate(next_page);
	}

    }
    
    public void showCourse(IRequestCycle cycle) {
        getMonitorSession().setCourse(getCurrentCourse());
        cycle.activate(getPage());
    }
    
    public CourseSelectionModel getStudentCourseList() {
            List monitorCourses = getMonitorSession().getStudent().getAllCourses();
            List studentCourses = getMonitorSession().getMonitoredStudent().getAllCourses();
            List acceptedCourses = new LinkedList();
            Course tempCourse = null;
            ListIterator studentCourseIterator = studentCourses.listIterator();
            while(studentCourseIterator.hasNext()) {
                tempCourse = (Course)studentCourseIterator.next();
                if(monitorCourses.contains(tempCourse));
                        acceptedCourses.add(tempCourse);
            }
            return new CourseSelectionModel(acceptedCourses,getPage().getLocale().getLanguage());
    }
    
    public boolean isMultipleCourses() {
        if(getStudentCourseList().getList().size() > 1)
            return true;
        return false;
    }
        
    public boolean isEditPersonalDeadline() {
        if(getEditingIndex() == getRoundIndex())
            return true;
        return false;
    }
    
    public boolean isEditExercise() {
        if(getEditingExerciseIndex() == getExerciseIndex() && getRound() == getMonitorSession().getRound())
            return true;
        return false;            
    }
    
    public String getRowStyle() {
        if((getExerciseIndex() % 2)==1)
            return getRowStyle2();
        else return getRowStyle1();
    }
    
    public void changeEmail(IRequestCycle cycle) {
        Student s = getMonitorSession().getMonitoredStudent();
        DataStorage ds = DataStorage.getInstance();
        if(!s.getEmail().equals("") && !s.getEmail().equals(getMonitorSession().getMonitor().getEmail()))
            ds.setStudentField(s.getId(), "email", s.getEmail());
        cycle.activate(this);
    }

}
