// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import org.apache.tapestry.html.BasePage;
import org.apache.tapestry.IRequestCycle;
import cexer.*;
import cexer.util.FingerprintHelper;
import cexer.util.Logging;

import java.security.NoSuchAlgorithmException;
import java.util.List;

public abstract class MonitorViewStudentSubmissionPage extends SecurePage {

    private Answer answer;

    public abstract MonitorSession getMonitorSession();

    public Exercise getExercise() {
	return getMonitorSession().getExercise();
    }

    public void setExercise(Exercise ex) {}

    public String getExerciseName() {
	return getExercise().getLocalizedName(getPage().getLocale().getLanguage());
    }

    public void setExerciseName(String s) {}

    public void setAnswer(Answer a) {
	answer = a;
    }
    
    public Answer getAnswer() {
	return answer;
    }
    
    public String getStatusMessage() {
        String message = null;
        switch(answer.getStatus()) {
        case DataStorage.OK:
            message = "";
            break;
        case DataStorage.AFTER_DEADLINE:
            message = "Submission after deadline.";
            break;
        case DataStorage.INVALID_FINGERPRINT:
            message = "Invalid fingerprint.";
            break;
        case DataStorage.AFTER_CLOSING:
            message = "Submission after course has closed.";
            break;
        case DataStorage.ALL_SUBMISSIONS_USED:
            message = "All submissions used.";
            break;
        case DataStorage.OK_MODIFIED:
            message = "Status OK. Points modified.";
            break;
        case DataStorage.AFTER_DEADLINE_MODIFIED:
            message = "Submission after deadline. Points modified.";
            break;
        case DataStorage.INVALID_FINGERPRINT_MODIFIED:
            message = "Invalid fingerprint. Points modified";
            break;
        case DataStorage.AFTER_CLOSING_MODIFIED:
            message = "Submission after course has closed. Points modified.";
            break;
        case DataStorage.ALL_SUBMISSIONS_USED_MODIFIED:
            message = "All submissions used. Points modified.";
            break;
        }
        return message;
    }

    public String getMd5Sum() {
	String md5 = "";
	MonitorSession session = getMonitorSession();
	String exerciseClass = "";
	List l = session.getExercise().getAppletParameters();
	for(int i = 0; i < l.size(); i++) {
	    AppletParameter a = (AppletParameter)l.get(i);
	    if("ex".equals(a.getName())) {
		exerciseClass = a.getValue();
		break;
	    }
	}
	System.out.println(""+session.getCourse().getId() + " " + session.getRound().getRoundNumber() + " " + session.getExercise().getNumber() + " " + session.getMonitoredStudent().getStudentNumber() + " " + session.getExercise().getMax() + " " + exerciseClass);
	try {
	    md5 = FingerprintHelper.calculateMD5(
						 FingerprintHelper.getFingerprint(""+session.getCourse().getId(),
                                 session.getRound().getRoundNumber(), 
                                 session.getExercise().getNumber(),
                                 session.getMonitoredStudent().getId(),
                                 session.getExercise().getMax(), exerciseClass));
	} catch(NoSuchAlgorithmException e) {
	    Logging.error(this, "Could not generate md5sum");
	}
	return md5;
    }

    public String getCodeBase() {
        if(getMonitorSession().getExercise().getExerciseTemplate() instanceof AppletExerciseTemplate)
            return ((AppletExerciseTemplate)getMonitorSession().getExercise().getExerciseTemplate()).getCodeBase();
        else return "";
    }

    public String getArchive() {
	return "matrix.jar";
    }

    public String getRunClass() {
	return "applications.trakla2.viewer.TestApplet.class";
    }

    public String getHeight() {
	return "200";
    }

    public String getWidth() {
	return "800";
    }
    
    public String getPort() {
        String port = System.getProperty("submissionviewer.server.port");
        if (port == null)
            port = "";
        return port;
    }

    public void detach() {
	super.detach();
	answer = null;
    }

}
