// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.util;

import java.util.HashMap;
import java.util.List;
import java.io.BufferedReader;
import java.util.LinkedList;

public class ExerciseDescriptionParser {
    public static final String INSTRUCTIONS_CLASS = "instructions";
    public static final String TASK_CLASS = "task";
    public static final String CODE_CLASS = "code";
    public static final String CODE_NAME_ATTRIBUTE = "name";
    private static final Object TAB_CLASS = "tab";
    
    private static final int TASK = 1;
    private static final int INSTRUCTIONS = 2;
    private static final int CODE = 3;
    private static final int TAB = 4;
    
    private int parserState;
    
    private String instructions;
    private String task;
    private String currentBlock;
    private String currentBlockName;
    private LinkedList codeBlockNames;
    private HashMap codeBlocks;        
    private List tabNames;
    private HashMap tabs;        
        
    /*
    public static void main(String args[]) {
        try {
            BufferedReader reader = new BufferedReader(new FileReader(args[0]));
            ExerciseDescriptionParser parser = new ExerciseDescriptionParser(reader);
            System.out.println("Instructions: " + parser.getInstructions());
            System.out.println("Task: " + parser.getTask());            
            System.out.println("Code1: " + parser.getCodeBlock("pseudo3"));
            System.out.println("Codeblocks: ");
            for (int i = 0; i < parser.getNumberOfCodeBlocks(); i++)
                System.out.println((String)parser.getCodeBlockName(i));
            
        }catch (Exception e) { System.out.println("Error: " + e);}
    }
    */
    
    public ExerciseDescriptionParser(BufferedReader reader) {
        currentBlock = "";
        currentBlockName = "";
        task = "";
        instructions = "";
        codeBlocks = new HashMap();
        codeBlockNames = new LinkedList();
        tabs = new HashMap();
        tabNames = new LinkedList();
        parserState = 0;
        
        try {
            String temp = reader.readLine();
            while(temp != null) {
                if(temp.toLowerCase().indexOf("<span") != -1 && temp.toLowerCase().indexOf("/>") != -1) {
                    boolean statechanged = parse(temp.substring(temp.toLowerCase().indexOf("<span") + 5, temp.toLowerCase().indexOf("/>")));
                    if(statechanged)
                        temp = temp.substring(temp.toLowerCase().indexOf("/>", temp.toLowerCase().indexOf("<span") + 5) + 2); //crop the <span/>-tag from input
                }
                switch(parserState) {
                case 0:
                    break;
                case TASK:
                    task = task + temp + "\n";
                    break;
                case INSTRUCTIONS:
                    instructions = instructions + temp + "\n";
                    break;
                case CODE:
                    currentBlock = currentBlock + temp + "\n";
                    break;
                case TAB:
                    currentBlock = currentBlock + temp + "\n";                    
                    break;
                }
                temp = reader.readLine();
            }
            if(currentBlockName != "" && parserState == CODE) { //Write last codeblock to hashmap when parsing is finished
                    codeBlocks.put(currentBlockName, currentBlock);
                    codeBlockNames.add(currentBlockName);
                    currentBlockName= "";
                    currentBlock="";
            } else if(currentBlockName != "" && parserState == TAB) { //Write last codeblock to hashmap when parsing is finished
                tabs.put(currentBlockName, currentBlock);
                tabNames.add(currentBlockName);
                currentBlockName= "";
                currentBlock="";
        }

        } catch (Exception e) {               
            }   
               
    }

    private boolean parse(String s) {
        String className = "";
        if(s.toLowerCase().indexOf("class=\"") != -1 && 
                s.substring(s.toLowerCase().indexOf("class=\"") + 7).indexOf("\"") != -1)
        {
            className= s.substring(s.toLowerCase().indexOf("class=\"") + 7,
                    s.indexOf("\"", s.toLowerCase().indexOf("class=\"") + 7));
        }
        
        if(className != "") {
            if(className.equals(INSTRUCTIONS_CLASS)) {
                processCurrentBlock();
                parserState = INSTRUCTIONS;
                return true;
            }
            if(className.equals(TASK_CLASS)) {
                processCurrentBlock();
                parserState = TASK;
                return true;
            }
            if(className.equals(CODE_CLASS) && s.toLowerCase().indexOf("name=\"") != -1) {
                processCurrentBlock();
                parserState = CODE;
                currentBlockName = s.substring(s.toLowerCase().indexOf("name=\"") + 6,
                        s.toLowerCase().indexOf("\"", s.toLowerCase().indexOf("name=\"") + 6));
                return true;
            }
            if (className.equals(TAB_CLASS) && s.toLowerCase().indexOf("name=\"") != -1) {
                processCurrentBlock();
                parserState = TAB;
                currentBlockName = s.substring(s.toLowerCase().indexOf("name=\"") + 6,
                        s.toLowerCase().indexOf("\"", s.toLowerCase().indexOf("name=\"") + 6));
                return true;              
            }
            return false;
        } return false;
    }
    
    private void processCurrentBlock() {
        if(currentBlockName != "" && parserState == CODE) {
            codeBlocks.put(currentBlockName, currentBlock);
            codeBlockNames.add(currentBlockName);
            currentBlockName= "";
            currentBlock="";
        } else if(currentBlockName != "" && parserState == TAB) {
            tabs.put(currentBlockName, currentBlock);
            tabNames.add(currentBlockName);
            currentBlockName= "";
            currentBlock="";
        }
    }

    public String getInstructions() {
        return instructions;
    }

    public String getTask() {
        return task;
    }
    
    public String getCodeBlockName(int index) {
        if (index < 0 || index >= getNumberOfCodeBlocks())
            return null;
        return (String)codeBlockNames.get(index);
    }
    
    public int getNumberOfCodeBlocks() {
        return codeBlockNames.size();
    }
    
    public String getCodeBlock(String blockName) {
        return (String)codeBlocks.get(blockName);
    }
    
    public HashMap getCodeBlocks() {
        return codeBlocks;       
    }
    
    public LinkedList getCodeBlockNames() {
        return codeBlockNames;
    }
    
    public String getTabName(int index) {
        if (index < 0 || index >= getNumberOfTabs())
            return null;
       return (String)tabNames.get(index);
    }
    
    public int getNumberOfTabs() {
        return tabNames.size();
    }
    
    public String getTab(String tabName) {
        return (String)tabs.get(tabName);
    }
    
    public HashMap getTabs() {
        return tabs;       
    }
    
    public List getTabNames() {
        return tabNames;
    }

}
