// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.util;

import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.TimeZone;

import org.apache.tapestry.form.IPropertySelectionModel;


public class TimeZoneSelectionModel implements IPropertySelectionModel {

    private List zoneList;
    
    public static void main(String args[]) {               
        TimeZoneSelectionModel model = new TimeZoneSelectionModel();
        
        for (int i=0; i< model.getZoneList().size(); i++) {
          System.out.println(model.getZoneList().get(i));            
        }
        
        for (int i=0; i<model.getOptionCount(); i++) {
            System.out.println(model.getLabel(i));            
        }
    }
    
    public List getZoneList() {
        return zoneList;
    }
    
    public TimeZoneSelectionModel() {        
        Date today = new Date();    
        String[] zoneIds = TimeZone.getAvailableIDs();
        List tempList = new LinkedList();
        
        for (int i=0; i<zoneIds.length; i++) {
            TimeZone tz = TimeZone.getTimeZone(zoneIds[i]);
            if(!tz.getDisplayName(tz.inDaylightTime(today), TimeZone.SHORT).contains("GMT"))
                tempList.add(tz.getID());
        }
        
        Set set = new LinkedHashSet();
        set.addAll(tempList);
        zoneList = new ArrayList(set);
    }
    
    public String getLabel(int index) {
        Date today = new Date();
        TimeZone tz = TimeZone.getTimeZone((String)zoneList.get(index));
        int offset = tz.getOffset(today.getTime());
        //int offset = tz.getRawOffset();
        //Date tempDate = new Date(Math.abs(offset));
        //DateFormat df = new SimpleDateFormat("HH:mm");
        
        int hour = offset / (60*60*1000);
        String hourString = "";
        if(hour < 0)
            hourString += "-";
        else hourString += "+";
        
        if(Math.abs(hour) < 10)
            hourString += "0";
        hourString += Math.abs(hour);
        
        int min = Math.abs(offset / (60*1000)) % 60;
        String minString = "";
        if(min < 10)
            minString += "0";
        minString += min;
                
        return "GMT" + hourString + ":" + minString + " " + tz.getID();        
    }

    public Object getOption(int index) {
        return zoneList.get(index);
    }

    public int getOptionCount() {
        return zoneList.size();
    }

    public String getValue(int index) {
        return Integer.toString(index);
    }

    public Object translateValue(String value) {
        return getOption(Integer.parseInt(value));
    }

}
