package cexer.util.mceparser;

import java.io.IOException;
import java.io.StringReader;
import java.util.LinkedList;
import java.util.List;
import java.util.Stack;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import cexer.Choice;
import cexer.ChoiceGroup;
import cexer.Question;
import cexer.QuestionGroup;

public class MceParser implements ContentHandler {
    private Stack elements;
    private List questionGroups;
    private String lang;

    public void characters(char[] ch, int start, int length)
            throws SAXException {
        if (!elements.isEmpty() && elements.peek() instanceof StringBuffer) {
            StringBuffer sb = (StringBuffer) elements.peek();
            for (int i = start; i < start + length; i++){
                sb.append(toHtmlEntity(ch[i]));
            }
        }
    }
    
    private String toHtmlEntity(char ch){
    	if (ch == '<'){
    		return "&lt;";
    	}
    	else if (ch == '>'){
    		return "&gt;";
    	}
    	else if (ch == '&'){
    		return "&amp;";
    	}
    	
    	// non-latin1 characters
    	else if ((int)ch > 255){
        	return "&#" + (int)ch + ";";
    	}
    	return "" + ch;
    }

    public void endDocument() throws SAXException {
        // TODO Auto-generated method stub
//        System.out.println(((QuestionGroup) questionGroups.get(0)).toXml());
    }

    public void endElement(String uri, String localName, String qName)
            throws SAXException {
        //System.out.println("end elment: " + localName);
        if (localName.equals(MceConstants.MCEXERCISE_TAG_NAME))
            endMcExerciseElement(uri, localName, qName);
        else if (localName.equals(MceConstants.QUESTIONGROUP_TAG_NAME))
            endQuestionGroupElement(uri, localName, qName);
        else if (localName.equals(MceConstants.QUESTION_TAG_NAME))
            endQuestionElement(uri, localName, qName);
        else if (localName.equals(MceConstants.DESCRIPTION_TAG_NAME))
            endDescriptionElement(uri, localName, qName);
        else if (localName.equals(MceConstants.POSITIVE_FEEDBACK_TAG_NAME))
            endPositiveFeedbackElement(uri, localName, qName);
        else if (localName.equals(MceConstants.NEGATIVE_FEEDBACK_TAG_NAME))
            endNegativeFeedbackElement(uri, localName, qName);
        else if (localName.equals(MceConstants.SELECTED_FEEDBACK_TAG_NAME))
            endSelectedFeedbackElement(uri, localName, qName);
        else if (localName.equals(MceConstants.UNSELECTED_FEEDBACK_TAG_NAME))
            endUnselectedFeedbackElement(uri, localName, qName);
        else if (localName.equals(MceConstants.CHOICEGROUP_TAG_NAME))
            endChoiceGroupElement(uri, localName, qName);
        else if (localName.equals(MceConstants.CHOICE_TAG_NAME))
            endChoiceElement(uri, localName, qName);
//        else
//            System.out.println("end elment: " + localName);
    }

    private void endMcExerciseElement(String uri, String localName, String name) {

    }

    private void endQuestionGroupElement(String uri, String localName, String name) {
        elements.pop();
    }

    private void endQuestionElement(String uri, String localName, String name) {
        elements.pop();
    }

    private void endDescriptionElement(String uri, String localName, String name) {
        if (elements.peek() instanceof StringBuffer) {
            StringBuffer sb = (StringBuffer) elements.pop();
            if (elements.peek() instanceof Question){
                ((Question) elements.peek()).addDescription(sb.toString().trim(), lang);
            }
            else if (elements.peek() instanceof Choice){
            	((Choice) elements.peek()).addDescription(sb.toString().trim(), lang);
            }
        }
    }

    private void endPositiveFeedbackElement(String uri, String localName, String name) {
        if (elements.peek() instanceof StringBuffer) {
            StringBuffer sb = (StringBuffer) elements.pop();
            if (elements.peek() instanceof Question)
                ((Question) elements.peek()).addPositiveFeedback(sb.toString().trim(), lang);
        }
    }

    private void endNegativeFeedbackElement(String uri, String localName, String name) {
        if (elements.peek() instanceof StringBuffer) {
            StringBuffer sb = (StringBuffer) elements.pop();
            if (elements.peek() instanceof Question)
                ((Question) elements.peek()).addNegativeFeedback(sb.toString().trim(), lang);
        }
    }
    
    private void endSelectedFeedbackElement(String uri, String localName, String name) {
        if (elements.peek() instanceof StringBuffer) {
            StringBuffer sb = (StringBuffer) elements.pop();
            if (elements.peek() instanceof Choice)
                ((Choice) elements.peek()).addSelectedFeedback(sb.toString().trim(), lang);
        }
    }

    private void endUnselectedFeedbackElement(String uri, String localName, String name) {
        if (elements.peek() instanceof StringBuffer) {
            StringBuffer sb = (StringBuffer) elements.pop();
            if (elements.peek() instanceof Choice)
                ((Choice) elements.peek()).addUnselectedFeedback(sb.toString().trim(), lang);
        }
    }

    private void endChoiceGroupElement(String uri, String localName, String name) {
        elements.pop();
    }

    private void endChoiceElement(String uri, String localName, String name) {
        elements.pop();
    }

    public void endPrefixMapping(String prefix) throws SAXException {
        // TODO Auto-generated method stub

    }

    public void ignorableWhitespace(char[] ch, int start, int length)
            throws SAXException {
        // TODO Auto-generated method stub

    }

    public void processingInstruction(String target, String data)
            throws SAXException {
        // TODO Auto-generated method stub

    }

    public void setDocumentLocator(Locator locator) {
        // TODO Auto-generated method stub

    }

    public void skippedEntity(String name) throws SAXException {
        // TODO Auto-generated method stub

    }

    public void startDocument() throws SAXException {
//        System.out.println("Start document");
        elements = new Stack();
        questionGroups = new LinkedList();
    }

    public void startElement(String uri, String localName, String qName,
            Attributes atts) throws SAXException {
        //System.out.println(localName + "\t" + elements.size());
        if (localName.equals(MceConstants.MCEXERCISE_TAG_NAME))
            startMcExerciseElement(uri, localName, qName, atts);
        else if (localName.equals(MceConstants.QUESTIONGROUP_TAG_NAME))
            startQuestionGroupElement(uri, localName, qName, atts);
        else if (localName.equals(MceConstants.QUESTION_TAG_NAME))
            startQuestionElement(uri, localName, qName, atts);
        else if (localName.equals(MceConstants.DESCRIPTION_TAG_NAME))
            startDescriptionElement(uri, localName, qName, atts);
        else if (localName.equals(MceConstants.POSITIVE_FEEDBACK_TAG_NAME))
            startPositiveFeedbackElement(uri, localName, qName, atts);
        else if (localName.equals(MceConstants.NEGATIVE_FEEDBACK_TAG_NAME))
            startNegativeFeedbackElement(uri, localName, qName, atts);
        else if (localName.equals(MceConstants.SELECTED_FEEDBACK_TAG_NAME))
            startSelectedFeedbackElement(uri, localName, qName, atts);
        else if (localName.equals(MceConstants.UNSELECTED_FEEDBACK_TAG_NAME))
            startUnselectedFeedbackElement(uri, localName, qName, atts);
        else if (localName.equals(MceConstants.CHOICEGROUP_TAG_NAME))
            startChoiceGroupElement(uri, localName, qName, atts);
        else if (localName.equals(MceConstants.CHOICE_TAG_NAME))
            startChoiceElement(uri, localName, qName, atts);
//        else
//            System.out.println("Start elment: " + localName);
    }

    private void startMcExerciseElement(String uri, String localName, String name, Attributes atts) {

    }

    private void startQuestionGroupElement(String uri, String localName, String name, Attributes atts) {
        QuestionGroup qg = new QuestionGroup();
        questionGroups.add(qg);
        elements.push(qg);
        try {
            int visible = Integer.parseInt(atts.getValue(MceConstants.VISIBLE_QUESTIONS_ATTRIBUTE_NAME));
            qg.setNumberOfVisibleQuestions(visible);
        } catch (NumberFormatException e) {
            System.out.println("Exception in MceParser: " + e.getMessage());
        }
    }

    private void startQuestionElement(String uri, String localName, String name, Attributes atts) {
        if (elements.peek() instanceof QuestionGroup) {
            Question q = new Question();
            ((QuestionGroup) elements.peek()).addQuestion(q);
            elements.push(q);
            int minSelected = 1;
            try {
                minSelected = Integer.parseInt(atts.getValue(MceConstants.MIN_SELECTED_ATTRIBUTE_NAME));
            } catch (NumberFormatException e) {
                System.out.println("Exception in MceParser: " + e.getMessage());
            }
            q.setMinSelected(minSelected);
            int maxSelected = 1;
            try {
                maxSelected = Integer.parseInt(atts.getValue(MceConstants.MAX_SELECTED_ATTRIBUTE_NAME));
            } catch (NumberFormatException e) {
                System.out.println("Exception in MceParser: " + e.getMessage());
            }
            q.setMaxSelected(maxSelected);
            if (atts.getIndex(MceConstants.OPTIONAL_ATTRIBUTE_NAME) != -1) {
                boolean opt = Boolean.parseBoolean(atts.getValue(MceConstants.OPTIONAL_ATTRIBUTE_NAME));
                q.setOptional(opt);
            }
            if (atts.getIndex(MceConstants.PREFORMATTED) != -1) {
                boolean pf = Boolean.parseBoolean(atts.getValue(MceConstants.PREFORMATTED));
                q.setPreformatted(pf);
            }
            if (atts.getIndex(MceConstants.CHOICES_LAYOUT) != -1) {
                boolean columns = Boolean.parseBoolean(atts.getValue(MceConstants.CHOICES_LAYOUT));
                q.setColumnLayout(columns);
            }
           
        } else {
            System.out.println("Error in MceParser.startQuestionElement");
        }
    }

    private void startDescriptionElement(String uri, String localName, String name, Attributes atts) {
        elements.push(new StringBuffer());
        lang = atts.getValue(MceConstants.LANG_ATTRIBUTE_NAME);
    }

    private void startPositiveFeedbackElement(String uri, String localName, String name, Attributes atts) {
        elements.push(new StringBuffer());
        lang = atts.getValue(MceConstants.LANG_ATTRIBUTE_NAME);
    }

    private void startNegativeFeedbackElement(String uri, String localName, String name, Attributes atts) {
        elements.push(new StringBuffer());
        lang = atts.getValue(MceConstants.LANG_ATTRIBUTE_NAME);
    }
    
    private void startSelectedFeedbackElement(String uri, String localName, String name, Attributes atts) {
        elements.push(new StringBuffer());
        lang = atts.getValue(MceConstants.LANG_ATTRIBUTE_NAME);
    }

    private void startUnselectedFeedbackElement(String uri, String localName, String name, Attributes atts) {
        elements.push(new StringBuffer());
        lang = atts.getValue(MceConstants.LANG_ATTRIBUTE_NAME);
    }

    private void startChoiceElement(String uri, String localName, String name, Attributes atts) {
        if (elements.peek() instanceof ChoiceGroup) {
            Choice c = new Choice();
            ((ChoiceGroup) elements.peek()).addChoice(c);
            elements.push(c);
            int selPoints = 0;
            try {
                 selPoints = Integer.parseInt(atts.getValue(MceConstants.SELECTED_POINTS_ATTRIBUTE_NAME));
            } catch (NumberFormatException e) {
                System.out.println("Exception in MceParser: " + e.getMessage());
            }
            c.setSelectedPoints(selPoints);
            int unselPoints = 0;
            try {
                unselPoints = Integer.parseInt(atts.getValue(MceConstants.UNSELECTED_POINTS_ATTRIBUTE_NAME));
            } catch (NumberFormatException e) {
                System.out.println("Exception in MceParser: " + e.getMessage());
            }
            c.setUnselectedPoints(unselPoints);
            if (atts.getIndex(MceConstants.OPTIONAL_ATTRIBUTE_NAME) != -1) {
                boolean opt = Boolean.parseBoolean(atts.getValue(MceConstants.OPTIONAL_ATTRIBUTE_NAME));
                c.setOptional(opt);
            }
        } else {
            System.out.println("Error in MceParser.startChoiceElement");
        }
    }

    private void startChoiceGroupElement(String uri, String localName, String name, Attributes atts) {
        ChoiceGroup cg = new ChoiceGroup();
        //questionGroups.add(qg);
        ((Question) elements.peek()).addChoiceGroup(cg);
        elements.push(cg);
        try {
            int visible = Integer.parseInt(atts.getValue(MceConstants.VISIBLE_CHOICES_ATTRIBUTE_NAME));
            cg.setNumberOfVisibleChoices(visible);
        } catch (NumberFormatException e) {
            System.out.println("Exception in MceParser: " + e.getMessage());
        }
    }

    public void startPrefixMapping(String prefix, String uri)
            throws SAXException {
        // TODO Auto-generated method stub

    }

    public static void main(String[] args) throws IOException, SAXException {
        MceParser xp = new MceParser();
        XMLReader parser = XMLReaderFactory.createXMLReader();
        parser.setContentHandler(xp);
        parser.parse("/home/vkaravir/projects/eclipse-workspace/T2Multiple/exer.xml");
    }

    public void parse(String exerciseXml) throws SAXException, IOException {
    	
    	
    	
        XMLReader parser = XMLReaderFactory.createXMLReader();
        parser.setContentHandler(this);
        parser.parse(new InputSource(new StringReader(exerciseXml)));
    }

    public List getQuestionGroups() {
        return questionGroups;
    }
}
