--
-- PostgreSQL database dump
--

SET default_with_oids = false;

--
-- Name: appletexercise; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE appletexercise (
    code character varying(255) NOT NULL,
    height smallint NOT NULL,
    width smallint NOT NULL,
    hintpage character varying(255),
    codebase character varying(255),
    archive character varying(255),
    eid integer NOT NULL,
    CONSTRAINT exercise_height CHECK ((height > 0)),
    CONSTRAINT exercise_id CHECK ((eid >= 0)),
    CONSTRAINT exercise_width CHECK ((width > 0))
);

--
-- Name: appletparameter; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE appletparameter (
    value character varying(255) NOT NULL,
    name character varying(255) NOT NULL,
    "type" character(1),
    id integer,
    CONSTRAINT appletparameter_type CHECK (((("type" = 'c'::bpchar) OR ("type" = 'e'::bpchar)) OR ("type" = 'r'::bpchar)))
);

--
-- Name: appletstrings; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE appletstrings (
    eid integer,
    lang character varying(3),
    cid integer,
    name text,
    value text
);

--
-- Name: choice; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE choice (
    id serial NOT NULL,
    selectedpoints integer,
    unselectedpoints integer
);

--
-- Name: choicedescription; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE choicedescription (
    choice_id integer NOT NULL,
    description text,
    lang character varying(3),
    positive_feedback text,
    negative_feedback text
);

--
-- Name: choicegroup; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE choicegroup (
    id serial NOT NULL,
    nr_of_visible_choices integer
);

--
-- Name: choicestogroups; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE choicestogroups (
    choicegroup_id integer,
    choice_id integer,
    condition smallint DEFAULT 0 NOT NULL,
    CONSTRAINT check_condition CHECK (((condition = 0) OR (condition = 1)))
);

--
-- Name: course; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE course (
    code character varying(20) NOT NULL,
    id integer NOT NULL,
    minpoints numeric DEFAULT 0,
    open timestamp with time zone,
    "close" timestamp with time zone,
    organization_id integer,
    organizer_uid character varying(12),
    timezone character varying(32) DEFAULT 'GMT'::character varying NOT NULL,
    CONSTRAINT course_id CHECK ((id >= 0)),
    CONSTRAINT course_minpoints CHECK ((minpoints >= (0)::numeric))
);

--
-- Name: coursedescription; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE coursedescription (
    cid integer,
    name character varying(255),
    lang character varying(3),
    description text,
    link character varying(1024),
    help text,
    dateformat character varying(32) DEFAULT 'dd MMM. yyyy HH:mm:ss'::character varying NOT NULL,
    ebookurl text
);

--
-- Name: coursegrading; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE coursegrading (
    cid integer,
    grade character varying(20),
    minpoints integer,
    CONSTRAINT coursegrading_minpoints CHECK ((minpoints >= 0))
);

--
-- Name: eventlog; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE eventlog (
    "timestamp" timestamp without time zone DEFAULT now(),
    "type" integer,
    description text
);

--
-- Name: exercise; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE exercise (
    id integer NOT NULL,
    "type" smallint DEFAULT (-1) NOT NULL
);

--
-- Name: exercisedescription; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE exercisedescription (
    description text NOT NULL,
    name character varying(255) NOT NULL,
    lang character varying(3) NOT NULL,
    eid integer,
    cid integer DEFAULT 0
);

--
-- Name: exercises; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE exercises (
    rid integer NOT NULL,
    eid integer,
    number integer NOT NULL,
    maxpoints smallint,
    maxsubs smallint DEFAULT 32767,
    CONSTRAINT exercises_maxpoints CHECK ((maxpoints >= 0)),
    CONSTRAINT exercises_maxsubs CHECK ((maxsubs >= 0))
);

--
-- Name: jhaveexercise; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE jhaveexercise (
    eid integer,
    jhaveurl character varying(512) NOT NULL,
    jhaveid character varying(128) NOT NULL
);

--
-- Name: link; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE link (
    name character varying(255) NOT NULL,
    target character varying(255) NOT NULL
);

--
-- Name: multiplechoiceexercisestogroups; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE multiplechoiceexercisestogroups (
    eid integer NOT NULL,
    questiongroup_id integer DEFAULT 0 NOT NULL
);

--
-- Name: organization; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE organization (
    id integer NOT NULL,
    shibid text,
    CONSTRAINT organization_id CHECK ((id >= 0))
);

--
-- Name: organizationdescription; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE organizationdescription (
    orgid integer NOT NULL,
    lang character varying(3) NOT NULL,
    acronym character varying(16),
    name character varying(255),
    link character varying(1024),
    description text
);

--
-- Name: parameter; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE parameter (
    value character varying(255) NOT NULL,
    name character varying(255) NOT NULL,
    "type" character(1),
    id integer
);

--
-- Name: personaldeadline; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE personaldeadline (
    userid character varying(255) NOT NULL,
    rid integer NOT NULL,
    deadline timestamp with time zone NOT NULL,
    grantedby character varying(255) NOT NULL
);

--
-- Name: question; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE question (
    id serial NOT NULL,
    min_selected integer,
    max_selected integer,
    preformatted integer DEFAULT 0,
    choices_layout integer DEFAULT 0,
    CONSTRAINT check_min_and_max CHECK ((((max_selected >= min_selected) AND (min_selected >= 0)) AND (max_selected >= 1)))
);

--
-- Name: questiondescription; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE questiondescription (
    question_id integer NOT NULL,
    description text,
    lang character varying(3),
    positive_feedback text,
    negative_feedback text
);

--
-- Name: questiongroup; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE questiongroup (
    id serial NOT NULL,
    nr_of_visible_questions integer
);

--
-- Name: questionstochoicegroups; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE questionstochoicegroups (
    question_id integer NOT NULL,
    choicegroup_id integer
);

--
-- Name: questionstogroups; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE questionstogroups (
    questiongroup_id integer NOT NULL,
    question_id integer,
    condition smallint NOT NULL,
    CONSTRAINT check_condition CHECK (((condition = 0) OR (condition = 1)))
);

--
-- Name: recoverpassword; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE recoverpassword (
    id character varying(255) NOT NULL,
    hash character varying(255) NOT NULL
);

--
-- Name: registrations; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE registrations (
    userid character varying(255),
    cid integer,
    userpriviledge integer DEFAULT 0
);

--
-- Name: results; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE results (
    points smallint NOT NULL,
    tstamp timestamp with time zone NOT NULL,
    userid character varying(255),
    eid integer,
    cid integer,
    rid integer,
    status integer DEFAULT 0,
    filename text,
    CONSTRAINT results_points CHECK ((points >= 0))
);

--
-- Name: round; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE round (
    cid integer,
    id integer NOT NULL,
    deadline timestamp with time zone NOT NULL,
    min numeric DEFAULT 0,
    number integer NOT NULL,
    makeupfactor numeric,
    open timestamp with time zone,
    CONSTRAINT round_makeupfactor CHECK ((((0)::numeric <= makeupfactor) AND (makeupfactor <= (1)::numeric))),
    CONSTRAINT round_min CHECK ((min >= (0)::numeric))
);

--
-- Name: rounddescription; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE rounddescription (
    rid integer,
    name character varying(255),
    lang character varying(3),
    description text
);

--
-- Name: users; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE users (
    id character varying(255) NOT NULL,
    "password" character varying(32),
    randomseed bigint,
    email character varying(255),
    name character varying(255),
    lang character varying(3),
    defcourse integer DEFAULT 0,
    studentnumber character varying(12),
    peerwise_key character varying(32)
);

--
-- Name: usershibinfo; Type: TABLE; Schema: public; Owner: trakla2; Tablespace: 
--

CREATE TABLE usershibinfo (
    id character varying(255) NOT NULL,
    studentnumber character varying(12),
    commonname character varying(256),
    displayname character varying(128),
    givenname character varying(128),
    surname character varying(128),
    email character varying(320),
    gender character(1) DEFAULT 'u'::bpchar,
    organization character varying(128),
    organizationname character varying(255),
    lang character varying(3),
    CONSTRAINT usershibinfo_gender CHECK ((((gender = 'm'::bpchar) OR (gender = 'f'::bpchar)) OR (gender = 'u'::bpchar)))
);

--
-- Name: applet_strings_key; Type: CONSTRAINT; Schema: public; Owner: trakla2; Tablespace: 
--

ALTER TABLE ONLY appletstrings
    ADD CONSTRAINT applet_strings_key UNIQUE (eid, lang, cid, name);

--
-- Name: course_pkey; Type: CONSTRAINT; Schema: public; Owner: trakla2; Tablespace: 
--

ALTER TABLE ONLY course
    ADD CONSTRAINT course_pkey PRIMARY KEY (id);

--
-- Name: exercise_pkey; Type: CONSTRAINT; Schema: public; Owner: trakla2; Tablespace: 
--

ALTER TABLE ONLY appletexercise
    ADD CONSTRAINT exercise_pkey PRIMARY KEY (eid);

--
-- Name: exercises_pkey; Type: CONSTRAINT; Schema: public; Owner: trakla2; Tablespace: 
--

ALTER TABLE ONLY exercises
    ADD CONSTRAINT exercises_pkey PRIMARY KEY (rid, number);

--
-- Name: pkey_question; Type: CONSTRAINT; Schema: public; Owner: trakla2; Tablespace: 
--

ALTER TABLE ONLY question
    ADD CONSTRAINT pkey_question PRIMARY KEY (id);

--
-- Name: pkey_questiongroup; Type: CONSTRAINT; Schema: public; Owner: trakla2; Tablespace: 
--

ALTER TABLE ONLY questiongroup
    ADD CONSTRAINT pkey_questiongroup PRIMARY KEY (id);

--
-- Name: pki_choice; Type: CONSTRAINT; Schema: public; Owner: trakla2; Tablespace: 
--

ALTER TABLE ONLY choice
    ADD CONSTRAINT pki_choice PRIMARY KEY (id);

--
-- Name: pki_choicegroup; Type: CONSTRAINT; Schema: public; Owner: trakla2; Tablespace: 
--

ALTER TABLE ONLY choicegroup
    ADD CONSTRAINT pki_choicegroup PRIMARY KEY (id);

--
-- Name: primary_key; Type: CONSTRAINT; Schema: public; Owner: trakla2; Tablespace: 
--

ALTER TABLE ONLY exercise
    ADD CONSTRAINT primary_key PRIMARY KEY (id);

--
-- Name: round_pkey; Type: CONSTRAINT; Schema: public; Owner: trakla2; Tablespace: 
--

ALTER TABLE ONLY round
    ADD CONSTRAINT round_pkey PRIMARY KEY (id);

--
-- Name: users_pkey; Type: CONSTRAINT; Schema: public; Owner: trakla2; Tablespace: 
--

ALTER TABLE ONLY users
    ADD CONSTRAINT users_pkey PRIMARY KEY (id);

--
-- Name: usershibinfo_pkey; Type: CONSTRAINT; Schema: public; Owner: trakla2; Tablespace: 
--

ALTER TABLE ONLY usershibinfo
    ADD CONSTRAINT usershibinfo_pkey PRIMARY KEY (id);

--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY usershibinfo
    ADD CONSTRAINT "$1" FOREIGN KEY (id) REFERENCES users(id) ON DELETE CASCADE;


--
-- Name: appletexercise_eid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY appletexercise
    ADD CONSTRAINT appletexercise_eid_fkey FOREIGN KEY (eid) REFERENCES exercise(id) ON DELETE CASCADE;


--
-- Name: cidfk; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY coursegrading
    ADD CONSTRAINT cidfk FOREIGN KEY (cid) REFERENCES course(id);


--
-- Name: coursedescription_cid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY coursedescription
    ADD CONSTRAINT coursedescription_cid_fkey FOREIGN KEY (cid) REFERENCES course(id) ON DELETE CASCADE;


--
-- Name: eidfk; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY exercises
    ADD CONSTRAINT eidfk FOREIGN KEY (eid) REFERENCES exercise(id);


--
-- Name: eidfk; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY exercisedescription
    ADD CONSTRAINT eidfk FOREIGN KEY (eid) REFERENCES exercise(id);


--
-- Name: eidfk; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY results
    ADD CONSTRAINT eidfk FOREIGN KEY (eid) REFERENCES exercise(id);


--
-- Name: exercises_rid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY exercises
    ADD CONSTRAINT exercises_rid_fkey FOREIGN KEY (rid) REFERENCES round(id) ON DELETE CASCADE;


--
-- Name: exercises_rid_fkey1; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY exercises
    ADD CONSTRAINT exercises_rid_fkey1 FOREIGN KEY (rid) REFERENCES round(id);


--
-- Name: grantedbyidfk; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY personaldeadline
    ADD CONSTRAINT grantedbyidfk FOREIGN KEY (grantedby) REFERENCES users(id);


--
-- Name: jhaveexercise_eid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY jhaveexercise
    ADD CONSTRAINT jhaveexercise_eid_fkey FOREIGN KEY (eid) REFERENCES exercise(id) ON DELETE CASCADE;


--
-- Name: questionstochoicegroups_choicegroup_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY questionstochoicegroups
    ADD CONSTRAINT questionstochoicegroups_choicegroup_id_fkey FOREIGN KEY (choicegroup_id) REFERENCES choicegroup(id);


--
-- Name: questionstochoicegroups_question_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY questionstochoicegroups
    ADD CONSTRAINT questionstochoicegroups_question_id_fkey FOREIGN KEY (question_id) REFERENCES question(id);


--
-- Name: ref_chdescr_ch_id_to_id; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY choicedescription
    ADD CONSTRAINT ref_chdescr_ch_id_to_id FOREIGN KEY (choice_id) REFERENCES choice(id);


--
-- Name: ref_ctogrps_cgid_to_id; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY choicestogroups
    ADD CONSTRAINT ref_ctogrps_cgid_to_id FOREIGN KEY (choicegroup_id) REFERENCES choicegroup(id);


--
-- Name: ref_ctogrps_chid_to_id; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY choicestogroups
    ADD CONSTRAINT ref_ctogrps_chid_to_id FOREIGN KEY (choice_id) REFERENCES choice(id);


--
-- Name: ref_mce_eid_to_id; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY multiplechoiceexercisestogroups
    ADD CONSTRAINT ref_mce_eid_to_id FOREIGN KEY (eid) REFERENCES exercise(id);


--
-- Name: ref_mce_qgid_to_id; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY multiplechoiceexercisestogroups
    ADD CONSTRAINT ref_mce_qgid_to_id FOREIGN KEY (questiongroup_id) REFERENCES questiongroup(id);


--
-- Name: ref_qdescr_qid_to_id; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY questiondescription
    ADD CONSTRAINT ref_qdescr_qid_to_id FOREIGN KEY (question_id) REFERENCES question(id);


--
-- Name: ref_qtoqgrps_qgrp_id_to_id; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY questionstogroups
    ADD CONSTRAINT ref_qtoqgrps_qgrp_id_to_id FOREIGN KEY (questiongroup_id) REFERENCES questiongroup(id);


--
-- Name: ref_qtoqgrps_qid_to_id; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY questionstogroups
    ADD CONSTRAINT ref_qtoqgrps_qid_to_id FOREIGN KEY (question_id) REFERENCES question(id);


--
-- Name: registrations_cid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY registrations
    ADD CONSTRAINT registrations_cid_fkey FOREIGN KEY (cid) REFERENCES course(id) ON DELETE CASCADE;


--
-- Name: registrations_userid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY registrations
    ADD CONSTRAINT registrations_userid_fkey FOREIGN KEY (userid) REFERENCES users(id) ON DELETE CASCADE;


--
-- Name: results_cid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY results
    ADD CONSTRAINT results_cid_fkey FOREIGN KEY (cid) REFERENCES course(id) ON DELETE CASCADE;


--
-- Name: results_userid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY results
    ADD CONSTRAINT results_userid_fkey FOREIGN KEY (userid) REFERENCES users(id) ON DELETE CASCADE;


--
-- Name: ridfk; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY personaldeadline
    ADD CONSTRAINT ridfk FOREIGN KEY (rid) REFERENCES round(id);


--
-- Name: ridfk; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY results
    ADD CONSTRAINT ridfk FOREIGN KEY (rid) REFERENCES round(id);


--
-- Name: round_cid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY round
    ADD CONSTRAINT round_cid_fkey FOREIGN KEY (cid) REFERENCES course(id) ON DELETE CASCADE;


--
-- Name: rounddescription_rid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY rounddescription
    ADD CONSTRAINT rounddescription_rid_fkey FOREIGN KEY (rid) REFERENCES round(id) ON DELETE CASCADE;


--
-- Name: useridfk; Type: FK CONSTRAINT; Schema: public; Owner: trakla2
--

ALTER TABLE ONLY personaldeadline
    ADD CONSTRAINT useridfk FOREIGN KEY (userid) REFERENCES users(id);


---
-- PostgreSQL database dump complete
--

